/* ==========================================================================
 * This file is part of the Trusted Platform Agent (TPA)
 *
 * Copyright (C) 2007-2010 Politecnico di Torino, Italy
 * TORSEC group -- http://security.polito.it
 *
 * Trusted Platform Agent (TPA) is copyrighted free software by Politecnico
 * di Torino (TORSEC group).  You can redistribute it and/or modify it under
 * either the terms of the LGPL (see COPYING.LESSER file), or the GPL (see
 * COPYING file).
 *
 * DISCLAIMER of WARRANTY
 *
 * The following software "Trusted Platform Agent (TPA)" developed for
 * Open_TC project (http://www.opentc.net), whilst this version is not
 * endorsed by Open_TC, is experimental and is provided "as is", and no
 * guarantee or warranty is given by Politecnico di Torino (TORSEC group),
 * that the software is fit for any particular purpose.
 * The user thereof uses the software at its sole risk and liability.
 * Politecnico di Torino (TORSEC group) shall have no obligation to maintain
 * or support this software.  Politecnico di Torino (TORSEC group) MAKES NO
 * EXPRESS OR IMPLIED WARRANTY OF ANY KIND REGARDING THIS SOFTWARE.
 * Politecnico di Torino SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED ON
 * CONTRACT, TORT OR ANY OTHER LEGAL THEORY, IN CONNECTION WITH OR
 * ARISING OUT OF THE FURNISHING, PERFORMANCE OR USE OF THIS SOFTWARE.
 * ==========================================================================
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "TPA_API.h"
#include "common.h"

void db_usage(char * programName) {
	printf("Usage: ");

	printf("%s db\n", programName);
	printf("optional parameters:\n");
	printf("  --path \tlocal storage path\n");
}

int db(int argc, char ** argv) {

	TPA_RESULT result = TPA_ERROR;
	TPA_CONTEXT *ctx = NULL;
	char *ls_path = NULL;

	if (argc == 1) {
		ls_path = NULL;
	} else if (argc == 3) {
		if (strcmp("--path", argv[1]) == 0) {
			ls_path = strdup(argv[2]);
		} else {
			db_usage(argv[0]);
			goto error;
		}
	} else {
		db_usage(argv[0]);
		goto error;
	}

	/* Allocate Context */
	if (TpaHL_CTX_allocate(&ctx) != TPA_SUCCESS)
		goto error;

	if (ls_path != NULL)
		if (TpaHL_CTX_set(ctx, CTX_LS_URI, strlen(ls_path), ls_path)
				!= TPA_SUCCESS)
			goto error;

	if (TpaHL_LS_create(ctx) != TPA_SUCCESS)
		goto error;

	result = TPA_SUCCESS;
	error: /* Free */
	if (ls_path)
		free(ls_path);
	TpaHL_CTX_freeMemory(ctx);
	return result;
}
