/* ==========================================================================
 * This file is part of the Trusted Platform Agent (TPA)
 *
 * Copyright (C) 2007-2010 Politecnico di Torino, Italy
 * TORSEC group -- http://security.polito.it
 *
 * Trusted Platform Agent (TPA) is copyrighted free software by Politecnico
 * di Torino (TORSEC group).  You can redistribute it and/or modify it under
 * either the terms of the LGPL (see COPYING.LESSER file), or the GPL (see
 * COPYING file).
 *
 * DISCLAIMER of WARRANTY
 *
 * The following software "Trusted Platform Agent (TPA)" developed for
 * Open_TC project (http://www.opentc.net), whilst this version is not
 * endorsed by Open_TC, is experimental and is provided "as is", and no
 * guarantee or warranty is given by Politecnico di Torino (TORSEC group),
 * that the software is fit for any particular purpose.
 * The user thereof uses the software at its sole risk and liability.
 * Politecnico di Torino (TORSEC group) shall have no obligation to maintain
 * or support this software.  Politecnico di Torino (TORSEC group) MAKES NO
 * EXPRESS OR IMPLIED WARRANTY OF ANY KIND REGARDING THIS SOFTWARE.
 * Politecnico di Torino SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED ON
 * CONTRACT, TORT OR ANY OTHER LEGAL THEORY, IN CONNECTION WITH OR
 * ARISING OUT OF THE FURNISHING, PERFORMANCE OR USE OF THIS SOFTWARE.
 * ==========================================================================
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "TPA_API.h"
#include "common.h"

void key_usage(char * programName) {
	printf("Usage: ");

	printf("%s key [SUBCOMMAND]\n", programName);

	printf("   createBK: creates a binding key");
	printf("\n\trequired parameters:\n");
	printf(
			"\t   -p keypwd  \tspecify the password of the binding key to create\n");
	printf("\t   -s srkpwd  \tspecify the SRK secret\n");
	printf("\toptional parameter:\n");
	printf("\t   -u uri     \tspecify URI for TPA localstorage\n");

	printf("\n   listBK: lists the binding keys saved in the local storage\n");
		printf("\toptional parameter:\n");
	printf("\t   -u uri     \tspecify URI for TPA localstorage\n");

	/*printf(
	 "\n   delDB [key uuid]: deletes an binding key from the local storage\n");*/

}

int bk_create(int argc, char **argv) {

	TPA_RESULT result = TPA_ERROR;
	TPA_CONTEXT *ctx = NULL;
	TPA_TPM *tpm = NULL;
	TPA_TPMKEY *key = NULL;

	int c = 0;
	int counter = 0;
	char *lstorage = NULL;

	/*
	 * This function will allocate all needed objects
	 */
	if (TpaHL_CTX_allocate(&ctx) != TPA_SUCCESS)
		goto error;

	if (TpaHL_TPM_allocate(&tpm) != TPA_SUCCESS)
		goto error;

	if (TpaHL_KEY_allocate(&key) != TPA_SUCCESS)
		goto error;

	/*
	 * Parsing parameters
	 */
	opterr = 0;

	while ((c = getopt(argc, argv, "p:s:u:")) != -1) {
		switch (c) {
		case 'p':
			if (TpaHL_KEY_set(key, KEY_TPMKEYSECRET, strlen(optarg), optarg)
					!= TPA_SUCCESS)
				goto error;
			counter++;
			break;
		case 's':
			if (TpaHL_TPM_set(tpm, TPM_SRKPWD, strlen(optarg), optarg)
					!= TPA_SUCCESS)
				goto error;
			counter++;
			break;
		case 'u':
			if (TpaHL_CTX_set(ctx, CTX_LS_URI, strlen(optarg), optarg)
					!= TPA_SUCCESS)
				goto error;
			lstorage = optarg;
			break;
		case '?':
			/*
			 * Unknown parameter here
			 */
			key_usage(argv[0]);
		default:
			/*
			 * Something went wrong here
			 */
			printf("Internal error!\n");
			goto error;
		}
	}

	if (counter != 2) {
		key_usage(argv[0]);
		goto error;
	}

	if (lstorage == NULL) {
		if (TpaHL_CTX_set(ctx, CTX_LS_URI, strlen(TPA_DEFAULT_LOCAL_STORAGE),
				TPA_DEFAULT_LOCAL_STORAGE) != TPA_SUCCESS)
			goto error;
	}

	/* Create Storage Master Key */
	if (TpaHL_SMK_Create(ctx, tpm) != TPA_SUCCESS) {
		printf("Sealing Master Key not created! Quit...\n");
		goto error;
	}

	if (TpaHL_KEY_CreateTpmBindingKey(ctx, tpm, key) == TPA_SUCCESS) {
		printf("Binding key created. ID: %d\n", key->tpmkey_id);
	} else {
		/*
		 * Something went wrong
		 */
		printf("Binding key not created!\n");
		goto error;
	}

	result = TPA_SUCCESS;

	error: /* Error label */
	TpaHL_KEY_freeMemory(key);
	TpaHL_TPM_freeMemory(tpm);
	TpaHL_CTX_freeMemory(ctx);

	return result;
}

int bk_list(int argc, char **argv) {
	TPA_CONTEXT *ctx = NULL;
	TPA_TPMKEY *keyList = NULL;
	int keyList_size = 0;
	int i = 0;
	int c = 0;
	char *lstorage = NULL;

	/* Allocate Context */
	if (TpaHL_CTX_allocate(&ctx) != TPA_SUCCESS)
		goto error;

	/*
	 * Parsing parameters
	 */
	opterr = 0;

	while ((c = getopt(argc, argv, "u:")) != -1) {
		switch (c) {
		case 'u':
			if (TpaHL_CTX_set(ctx, CTX_LS_URI, strlen(optarg), optarg)
					!= TPA_SUCCESS)
				goto error;
			lstorage = optarg;
			break;
		case '?':
			/*
			 * Unknown parameter here
			 */
			key_usage(argv[0]);
		default:
			/*
			 * Something went wrong here
			 */
			printf("Internal error!\n");
			goto error;
		}
	}

	if (lstorage == NULL) {
		/* Set Local Storage folder */
		if (TpaHL_CTX_set(ctx, CTX_LS_URI, strlen(TPA_DEFAULT_LOCAL_STORAGE),
				TPA_DEFAULT_LOCAL_STORAGE) != TPA_SUCCESS)
			goto error;
	}

	if (TpaHL_KEY_ListTpmBindingKey(ctx, &keyList, &keyList_size, 0)
			!= TPA_SUCCESS)
		goto error;

	if (keyList_size < 1) { /* be careful with key 0 */
		printf("No Binding keys found in db\n");
	} else {
		printf("Binding keys in db:\n");
		for (i = 0; i < keyList_size; i++) {
			if (keyList[i].tpmkey_id == 0)
				continue;
			printf("%d. uuid: %d\n", i, keyList[i].tpmkey_id);
		}
	}

	TpaHL_CTX_freeMemory(ctx);
	if (keyList)
		free(keyList);

	return TPA_SUCCESS;

	error: return TPA_ERROR;
}

int key(int argc, char ** argv) {
	TPA_RESULT res = TPA_ERROR;

	if (argc == 1) {
		key_usage(argv[0]);
		return TPA_ERROR;
	}
	if (strcmp("createBK", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = bk_create(argc, argv);
	} else if (strcmp("listBK", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = bk_list(argc, argv);
	} else {
		key_usage(argv[0]);
		return TPA_ERROR;
	}

	return res;
}

