/* ==========================================================================
 * This file is part of the Trusted Platform Agent (TPA)
 *
 * Copyright (C) 2007-2010 Politecnico di Torino, Italy
 * TORSEC group -- http://security.polito.it
 *
 * Trusted Platform Agent (TPA) is copyrighted free software by Politecnico
 * di Torino (TORSEC group).  You can redistribute it and/or modify it under
 * either the terms of the LGPL (see COPYING.LESSER file), or the GPL (see
 * COPYING file).
 *
 * DISCLAIMER of WARRANTY
 *
 * The following software "Trusted Platform Agent (TPA)" developed for
 * Open_TC project (http://www.opentc.net), whilst this version is not
 * endorsed by Open_TC, is experimental and is provided "as is", and no
 * guarantee or warranty is given by Politecnico di Torino (TORSEC group),
 * that the software is fit for any particular purpose.
 * The user thereof uses the software at its sole risk and liability.
 * Politecnico di Torino (TORSEC group) shall have no obligation to maintain
 * or support this software.  Politecnico di Torino (TORSEC group) MAKES NO
 * EXPRESS OR IMPLIED WARRANTY OF ANY KIND REGARDING THIS SOFTWARE.
 * Politecnico di Torino SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED ON
 * CONTRACT, TORT OR ANY OTHER LEGAL THEORY, IN CONNECTION WITH OR
 * ARISING OUT OF THE FURNISHING, PERFORMANCE OR USE OF THIS SOFTWARE.
 * ==========================================================================
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "TPA_API.h"
#include "TPA_Utils.h"
#include "common.h"

/*
 * TODO: FIXME, if -n number is not specified read and print all pcrs
 */

void pcr_usage(char * programName) {
	printf("Usage:\n");

	printf("%s pcr [SUBCOMMAND]\n", programName);

	printf("   read,\tread a PCR\n");
	printf("\toptions:\n");
	printf("\t   -n number\tspecify the number of the PCR to read\n");
	printf("   extend (EXPERIMENTAL),\textend a PCR\n");
	printf("\toptions:\n");
	printf("\t   -n number\tspecify the number of the PCR to extend\n");
	printf("\t   -v filename\tspecify the file used for extention - file will be hashed\n");
}


int pcr_extend(int argc, char ** argv) {
	TPA_RESULT result = TPA_ERROR;
	int c;
	char *filename = NULL;
	unsigned char *blob = NULL;
	unsigned int   blob_size = 0;

	TPA_PCR pcr = { 0 };

	if (argc == 1) {
		pcr_usage(argv[0]);
		exit(0);
	}
	while ((c = getopt(argc, argv, "n:v:")) != -1) {
		switch (c) {
		case 'n':
			pcr.pcr = atoi(optarg);
			break;
		case 'v':
			filename = optarg;
			if (filename == NULL) {
				pcr_usage(argv[0]);
				exit(0);
			}
			break;
		case '?':
			pcr_usage(argv[0]);
			exit(0);
			break;
		default:
			printf("Internal error!\n");
			exit(0);
			break;
		}
	}

	if (Tpa_BLOB_readFromFile(filename, &blob, &blob_size) != TPA_SUCCESS)
	{
		printf("TpaHL_BLOB_readFromFile failed\n");
		goto error;
	}

	if (TpaHL_PCR_Extend(&pcr, blob, blob_size) == TPA_SUCCESS) {
		printf("PCR %d extended\n", pcr.pcr);
	} else {
		printf("Error!\n");
		goto error;
	}

	result = TPA_SUCCESS;

	error:
	if(blob)
		free(blob);

	return result;
}

int pcr_read(int argc, char ** argv) {

	int i = 0;
	int c;

	TPA_PCR pcr = { 0 };

	while ((c = getopt(argc, argv, "n:")) != -1) {
		switch (c) {
		case 'n':
			pcr.pcr = atoi(optarg);
			break;
		case '?':
			pcr_usage(argv[0]);
			exit(0);
			break;
		default:
			printf("Internal error!\n");
			exit(0);
			break;
		}
	}

	if (TpaHL_PCR_Read(&pcr) == TPA_SUCCESS) {
		printf("PCR %d: ", pcr.pcr);

		for (i = 0; i < TPA_PCR_SIZE; i++) {
			printf("%02x ", pcr.value[i]);
		}
		printf("\n");
	} else {
		printf("Error!\n");
		return TPA_ERROR;
	}
	return TPA_SUCCESS;
}

int pcr(int argc, char ** argv) {
	TPA_RESULT res = TPA_ERROR;
	if (argc == 1) {
		pcr_usage(argv[0]);
		return TPA_ERROR;
	}
	if (strcmp("read", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = pcr_read(argc, argv);
	} else if (strcmp("extend", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = pcr_extend(argc, argv);
	} else {
		pcr_usage(argv[0]);
		return TPA_ERROR;
	}

	return res;
}

