/* ==========================================================================
 * This file is part of the Trusted Platform Agent (TPA)
 *
 * Copyright (C) 2007-2010 Politecnico di Torino, Italy
 * TORSEC group -- http://security.polito.it
 *
 * Trusted Platform Agent (TPA) is copyrighted free software by Politecnico
 * di Torino (TORSEC group).  You can redistribute it and/or modify it under
 * either the terms of the LGPL (see COPYING.LESSER file), or the GPL (see
 * COPYING file).
 *
 * DISCLAIMER of WARRANTY
 *
 * The following software "Trusted Platform Agent (TPA)" developed for
 * Open_TC project (http://www.opentc.net), whilst this version is not
 * endorsed by Open_TC, is experimental and is provided "as is", and no
 * guarantee or warranty is given by Politecnico di Torino (TORSEC group),
 * that the software is fit for any particular purpose.
 * The user thereof uses the software at its sole risk and liability.
 * Politecnico di Torino (TORSEC group) shall have no obligation to maintain
 * or support this software.  Politecnico di Torino (TORSEC group) MAKES NO
 * EXPRESS OR IMPLIED WARRANTY OF ANY KIND REGARDING THIS SOFTWARE.
 * Politecnico di Torino SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED ON
 * CONTRACT, TORT OR ANY OTHER LEGAL THEORY, IN CONNECTION WITH OR
 * ARISING OUT OF THE FURNISHING, PERFORMANCE OR USE OF THIS SOFTWARE.
 * ==========================================================================
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "TPA_API.h"

void usage(char *programName) {
	printf("Usage: ");
	printf("%s [COMMAND]\n", programName);
	printf("   ek   \tread public EK certificate (only IFX 1.2.1.2)\n");
	printf("   aik  \tcreate AIK key and certificate, using APKI protocol\n");
	printf("   key  \tcreate a tpm key\n");
	printf("   pcr  \tmanages the PCRs\n");
	printf("   db   \tdb initialization\n");
	printf("   pstore   \tprotected store capabilities\n");
	printf("   data   \tdata capabilities - bind\n");
	printf("   tpm   \ttpm admin commands\n");
/*	printf("   meas  \tmeasure logs commands\n");*/
	printf("   ra    \tremote attestation commands\n");
}

void ek_usage(char *);
void aik_usage(char *);
void pcr_usage(char *);

/*
 * EK functions
 */
int ek_read(int, char **);
int ek(int argc, char ** argv);

/*
 * AIK functions
 */
int aik_create(int, char **);
int aik(int, char **);

/*
 * KEY functions
 */
void key_usage(char *);
int bk_create(int, char **);
int bk_list(int, char **);
int key(int, char **);

/*
 * DB
 */
int db_usage(int, char **);
int db(int, char **);

/*
 * PSTORE
 */
int pstore_usage(int, char **);
int pstore(int, char **);

/*
 * TPM functions
 */
int tpm_usage(int, char **);
int tpm(int, char **);

/*
 * PCR functions
 */
unsigned char * ascii2hex(char *);
int pcr_read(int, char **);
int pcr_extend(int, char **);
int pcr(int, char **);

/*
 * DATA functions
 */
void data_usage(char *);
int data_bind(int, char **);
int data_unbind(int, char **);
int data(int, char **);

/*
 * RA functions
 */
void ra_usage(char *);
int ra_quote(int, char **);
int ra_verify(int, char **);
int ra(int, char **);

/*
 * MEASURE functions
 */
void meas_usage(char *);
int meas_clear(int, char **);
int meas_list(int, char **);
int meas_file(int, char **);
int meas_dlopen(int, char **);
int meas(int, char **);


/*
 * Main
 */
int main(int argc, char ** argv) {
	TPA_RESULT res = TPA_ERROR;

	if (argc == 1) {
		usage(argv[0]);
		return TPA_ERROR;
	}

	if (strcmp("ek", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = ek(argc, argv);
	} else if (strcmp("aik", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = aik(argc, argv);
	} else if (strcmp("key", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = key(argc, argv);
	} else if (strcmp("pcr", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = pcr(argc, argv);
	} else if (strcmp("db", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = db(argc, argv);
	} else if (strcmp("pstore", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = pstore(argc, argv);
	} else if (strcmp("data", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = data(argc, argv);
	} else if (strcmp("tpm", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = tpm(argc, argv);
	} else if (strcmp("ra", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = ra(argc, argv);
/*	} else if (strcmp("meas", argv[1]) == 0) {
		argc--;
		argv[1] = argv[0];
		argv++;
		res = meas(argc, argv); */
	} else {
		usage(argv[0]);
		return TPA_ERROR;
	}

	return res;
}
