/* ==========================================================================
 * This file is part of the Trusted Platform Agent (TPA)
 *
 * Copyright (C) 2007-2010 Politecnico di Torino, Italy
 * TORSEC group -- http://security.polito.it
 *
 * Trusted Platform Agent (TPA) is copyrighted free software by Politecnico
 * di Torino (TORSEC group).  You can redistribute it and/or modify it under
 * either the terms of the LGPL (see COPYING.LESSER file), or the GPL (see
 * COPYING file).
 *
 * DISCLAIMER of WARRANTY
 *
 * The following software "Trusted Platform Agent (TPA)" developed for
 * Open_TC project (http://www.opentc.net), whilst this version is not
 * endorsed by Open_TC, is experimental and is provided "as is", and no
 * guarantee or warranty is given by Politecnico di Torino (TORSEC group),
 * that the software is fit for any particular purpose.
 * The user thereof uses the software at its sole risk and liability.
 * Politecnico di Torino (TORSEC group) shall have no obligation to maintain
 * or support this software.  Politecnico di Torino (TORSEC group) MAKES NO
 * EXPRESS OR IMPLIED WARRANTY OF ANY KIND REGARDING THIS SOFTWARE.
 * Politecnico di Torino SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED ON
 * CONTRACT, TORT OR ANY OTHER LEGAL THEORY, IN CONNECTION WITH OR
 * ARISING OUT OF THE FURNISHING, PERFORMANCE OR USE OF THIS SOFTWARE.
 * ==========================================================================
 */
#ifndef TPA_API_H_
#define TPA_API_H_

#include "TPA_Common.h"
#include "TPA_Config.h"

/*
 * Defines
 */

/*
 * TPA types
 */

typedef struct {
	TPA_MODULE_KEY *key_mod;
	TPA_MODULE_MEASURE *measure_mod;
	TPA_LS *ls;
	TPA_TPM *tpm;
	/* backward compat */
	char *ls_uri;
	unsigned int ls_uri_size;
} TPA_CONTEXT;

/*
 * Parameters for Setters
 */
#define AIK_PUBKEY					 0
#define AIK_PRIVKEY					 1
#define AIK_CERT					 2
#define AIK_LABEL					 3
#define AIK_UUID					 4
#define AIK_AIKSECRET 				 5

#define CTX_LS_URI					10
#define CTX_JTOOLS_URI 				11

#define TPM_OWNERPWD 				20
#define TPM_SRKPWD 					21
#define TPM_EKCERT 					22
#define TPM_EKCERT_FILENAME			23

#define PCA_URI						30
#define PCA_PORT					31
#define PCA_AUTH					32
#define PCA_PCACERT					33
#define PCA_PCACERT_FILENAME		34
#define PCA_LOCAL					35
#define PCA_REMOTE					36

#define PCRSET_SIZE					50
#define PCRSET_PCR_ELEMENT			51
#define PCR_PCR_NUMBER				55
#define PCR_PCR_VALUE				56

#define KEY_PUBKEY					60
#define KEY_PRIVKEY					61
#define KEY_CERT					62
#define KEY_LABEL					63
#define KEY_UUID					64
#define KEY_TPMKEYSECRET			65

#define RA_NONCE					70
#define RA_AIK						71
#define RA_PCRS						72
#define RA_FLAGS					74
/*
 * Object Types
 */
#define TPA_OBJ_TYPE_KEY			 	0x0001
#define TPA_OBJ_TYPE_MEASURE		 	0x0002
#define TPA_OBJ_TYPE_MEASURESET		 	0x0004

#define TPA_OBJ_KEY_SUBTYPE_SMK			 0x0100
#define TPA_OBJ_KEY_SUBTYPE_STORAGE		 0x0200
#define TPA_OBJ_KEY_SUBTYPE_BINDING		 0x0400
#define TPA_OBJ_KEY_SUBTYPE_SEALING		 0x0800
#define TPA_OBJ_KEY_SUBTYPE_SIGNING		 0x1000
#define TPA_OBJ_KEY_SUBTYPE_ATTESTATION	 0x2000

/*
 * TPA functions
 */

/**
 * @addtogroup hlApi_tpa_ctx
 */
/*@{*/

TPA_RESULT TpaHL_CTX_allocate(TPA_CONTEXT **);
TPA_RESULT TpaHL_CTX_set(TPA_CONTEXT *, int, long, void *);
void TpaHL_CTX_freeMemory(TPA_CONTEXT *);

/*@}*/

/**
 * @addtogroup hlApi_tpa_ls
 */
/*@{*/

TPA_RESULT TpaHL_LS_create(TPA_CONTEXT *);

/*@}*/

/**
 * @addtogroup hlApi_tpa_tpm
 */
/*@{*/

TPA_RESULT TpaHL_TPM_allocate(TPA_TPM **);
TPA_RESULT TpaHL_TPM_set(TPA_TPM *, int, long, void *);
void TpaHL_TPM_freeMemory(TPA_TPM *);

TPA_RESULT TpaHL_TPM_IsOwnershipTaken(TPA_CONTEXT *, TPA_TPM *,
		TPA_TPM_OWNERSHIP_RESULT *);
TPA_RESULT TpaHL_TPM_TakeOwnership(TPA_CONTEXT *, TPA_TPM *);

/*@}*/

/**
 * @addtogroup hlApi_key
 */
/*@{*/

TPA_RESULT TpaHL_KEY_allocate(TPA_TPMKEY **);
TPA_RESULT TpaHL_KEY_set(TPA_TPMKEY *, int, long, void *);
void TpaHL_KEY_freeMemory(TPA_TPMKEY *);

TPA_RESULT
		TpaHL_KEY_CreateTpmBindingKey(TPA_CONTEXT *, TPA_TPM *, TPA_TPMKEY *);
TPA_RESULT TpaHL_KEY_ListTpmBindingKey(TPA_CONTEXT *, TPA_TPMKEY **, int *,
		unsigned int);
TPA_RESULT TpaHL_KEY_RetrieveTpmBindingKeyById(TPA_CONTEXT *, TPA_TPMKEY **,
		TPA_OBJ_ID);
TPA_RESULT TpaHL_KEY_DeleteTpmBindingKeyById(TPA_CONTEXT *, TPA_OBJ_ID);

/*@}*/

/**
 * @addtogroup hlApi_data_seal
 */
/*@{*/

TPA_RESULT TpaHL_DATA_BindBlob(TPA_CONTEXT *context, TPA_TPM *tpm,
		TPA_TPMKEY *key, unsigned char *plainblob, unsigned int plainblob_size,
		unsigned char **boundblob, unsigned int *boundblob_size);

TPA_RESULT TpaHL_DATA_UnbindBlob(TPA_CONTEXT *context, TPA_TPM *tpm,
		TPA_TPMKEY *key, unsigned char *boundblob, unsigned int boundblob_size,
		unsigned char **unboundblob, unsigned int *unboundblob_size);

/*@}*/

/**
 * @addtogroup hlApi_key_ek
 */
/*@{*/

TPA_RESULT TpaHL_EK_RetrieveCert(char *ekfile);
TPA_RESULT TpaHL_EK_CreateCert(TPA_TPM *tpm, TPA_PCA *pca);

/*@}*/

/**
 * @addtogroup hlApi_key_smk
 */
/*@{*/

TPA_RESULT TpaHL_SMK_Create(TPA_CONTEXT *ctx, TPA_TPM *tpm);

/*@}*/

/**
 * @addtogroup hlApi_key_gen
 */
/*@{*/

TPA_RESULT TpaHL_GenTpmKey_allocate(TPA_GENERIC_TPMKEY **gTpmKey);
TPA_RESULT TpaHL_GenTpmKey_set(TPA_GENERIC_TPMKEY *gTpmKey, int idx, long len,
		void *data);
void TpaHL_GenTpmKey_freeMemory(TPA_GENERIC_TPMKEY *gTpmKey);
TPA_RESULT TpaHL_GenTpmKey_writePubKeyToFile(char *filename,
		unsigned char *blob, int blob_len);
/*@}*/

/**
 * @addtogroup hlApi_key_pca
 */
/*@{*/

TPA_RESULT TpaHL_PCA_allocate(TPA_PCA **pca);
TPA_RESULT TpaHL_PCA_set(TPA_PCA *ca, int idx, long len, void *data);
void TpaHL_PCA_freeMemory(TPA_PCA *ca);

/*@}*/

/**
 * @addtogroup hlApi_key_aik
 */
/*@{*/

TPA_RESULT TpaHL_AIK_allocate(TPA_AIK **);
TPA_RESULT TpaHL_AIK_set(TPA_AIK *, int, long, void *);
void TpaHL_AIK_freeMemory(TPA_AIK *aik);
TPA_RESULT TpaHL_AIK_Create(TPA_CONTEXT *, TPA_TPM *, TPA_AIK *, TPA_PCA *);
TPA_RESULT TpaHL_AIK_Certify(TPA_CONTEXT *, TPA_TPM *, TPA_AIK *, TPA_PCA *);
TPA_RESULT TpaHL_AIK_List(TPA_CONTEXT *, TPA_AIK **, int *, unsigned int);
TPA_RESULT TpaHL_AIK_Retrieve(TPA_CONTEXT *context, TPA_AIK *aik);
TPA_RESULT TpaHL_AIK_RetrieveById(TPA_CONTEXT *context, TPA_AIK **aik,
		TPA_OBJ_ID id);
TPA_RESULT TpaHL_AIK_Delete(TPA_CONTEXT *, TPA_AIK *aik);
TPA_RESULT TpaHL_AIK_DeleteById(TPA_CONTEXT *, TPA_OBJ_ID);
TPA_RESULT TpaHL_AIK_Locate(TPA_CONTEXT *, TPA_AIK *, TPA_PCA *);
TPA_RESULT TpaHL_AIK_Revoke(TPA_CONTEXT *, TPA_AIK *, TPA_PCA *);
TPA_RESULT TpaHL_AIK_Validate(TPA_AIK *aik, TPA_PCA *ca,
		TPA_CERTIFICATE_ITEM **cert_list, TPA_VERIFICATION_RESULT *res);

/*@}*/
/**
 * @addtogroup hlApi_data_blob
 */
/*@{*/

/*@}*/

/**
 * @addtogroup hlApi_data_pcr
 */
/*@{*/

TPA_RESULT TpaHL_PCR_allocate(TPA_PCR **pcr);
TPA_RESULT TpaHL_PCR_set(TPA_PCR *pcr, int idx, long len, void *data);
void TpaHL_PCR_freeMemory(TPA_PCR *pcr);

TPA_RESULT TpaHL_PCR_Read(TPA_PCR *pcr);
TPA_RESULT TpaHL_PCR_Extend(TPA_PCR *pcr, unsigned char *blob,
		unsigned int blob_size);
TPA_RESULT TpaHL_PCR_ExtendH(TPA_PCR *pcr, unsigned char *digest,
		unsigned int digest_size);

TPA_RESULT TpaHL_PCRSet_allocate(TPA_PCR_SET **pcrSet);
void TpaHL_PCRSet_freeMemory(TPA_PCR_SET *pcrSet);
TPA_RESULT TpaHL_PCRSet_initialize(TPA_PCR_SET **pcrSet, int setDim,
		int pcrSize);
TPA_RESULT TpaHL_PCRSet_pcr(TPA_PCR_SET *pcrSet, int index,
		unsigned char* value);
TPA_RESULT TpaHL_PCRSet_Serialize(TPA_PCR_SET *selectedPcrs,
		unsigned char **blob, unsigned int *blob_size);
TPA_RESULT TpaHL_PCRSet_Deserialize(unsigned char *blob,
		unsigned int blob_size, unsigned char **blob_end,
		TPA_PCR_SET **selectedPcrs);

/*@}*/

/**
 * @addtogroup hlApi_data_seal
 */
/*@{*/

TPA_RESULT TpaHL_SealBlob(TPA_CONTEXT *context, TPA_TPM *tpm,
		unsigned char *plainblob, unsigned int plainblob_size,
		TPA_PCR_SET *selectedPcrs, unsigned char **sealedblob,
		unsigned int *sealedblob_size);

TPA_RESULT TpaHL_UnsealBlob(TPA_CONTEXT *context, TPA_TPM *tpm,
		unsigned char *sealedblob, unsigned int sealedblob_size,
		TPA_PCR_SET **selectedPcrs, unsigned char **unsealedblob,
		unsigned int *unsealedblob_size);

/*@}*/

/**
 * @addtogroup hlApi_rem_meas
 */
/*@{*/

void TpaHL_Measure_freeMemory(TPA_MEASURE *measure);
TPA_RESULT TpaHL_Measure_dlopen(TPA_CONTEXT *context, const char *filename,
		int flag, void **handle);
TPA_RESULT TpaHL_Measure_fopen(TPA_CONTEXT *context, const char *path,
		const char *mode, FILE **handle);
TPA_RESULT TpaHL_Measure_File(TPA_CONTEXT *context, const char *path,
		char *label, unsigned int label_len, TPA_MEASURE **measure);
TPA_RESULT TpaHL_Measure_Blob(TPA_CONTEXT *context, unsigned char *blob,
		unsigned int blob_size, char *label, unsigned int label_len,
		TPA_MEASURE **measure);
TPA_RESULT TpaHL_Measure_TargetPCR(TPA_CONTEXT *context, TPA_PCR_NUM pcr);

TPA_RESULT TpaHL_MeasureSet_allocate(TPA_MEASURE_SET **measures);
void TpaHL_MeasureSet_freeMemory(TPA_MEASURE_SET *measures);
TPA_RESULT TpaHL_MeasureSet_Serialize(TPA_MEASURE_SET *measures,
		unsigned char **blob, unsigned int *blob_size);
TPA_RESULT TpaHL_MeasureSet_Deserialize(unsigned char *blob,
		unsigned int blob_size, unsigned char **blob_end,
		TPA_MEASURE_SET **measures);

TPA_RESULT TpaHL_MeasureSet_RetrieveByPcrs(TPA_CONTEXT *context,
		TPA_PCR_SET *selectedPcrs, TPA_MEASURE_SET **measures);

TPA_RESULT TpaHL_Measure_DeleteAll(TPA_CONTEXT *context);

/*@}*/

/**
 * @addtogroup hlApi_rem_ra
 */
/*@{*/

TPA_RESULT TpaHL_RA_allocate(TPA_RA **ra);
TPA_RESULT TpaHL_RA_set(TPA_RA *ra, int idx, long len, void * data);
void TpaHL_RA_freeMemory(TPA_RA *ra);
TPA_RESULT TpaHL_RA_Serialize(TPA_RA *ra, unsigned char **blob,
		unsigned int *blob_size);
TPA_RESULT TpaHL_RA_Deserialize(unsigned char *blob, unsigned int blob_size,
		unsigned char **blob_end, TPA_RA **ra);

TPA_RESULT TpaHL_RA_Quote(TPA_CONTEXT *context, TPA_TPM *tpm, TPA_AIK *aik,
		TPA_PCR_SET *selectedPcrs, unsigned char *raNonce,
		unsigned int raNonce_size, TPA_RA *raResult);
TPA_RESULT TpaHL_RA_Verify(TPA_CONTEXT *context, TPA_RA *ra, TPA_AIK **aik,
		unsigned char **raNonce, unsigned int *raNonce_size,
		TPA_PCR_SET **pcrs, TPA_MEASURE_SET **measures,
		TPA_VERIFICATION_RESULT *res);

/*@}*/

/**
 * @addtogroup hlApi_rem_ra
 */
/*@{*/

TPA_RESULT TpaHL_RA_allocate(TPA_RA **ra);
TPA_RESULT TpaHL_RA_set(TPA_RA *ra, int idx, long len, void * data);
void TpaHL_RA_freeMemory(TPA_RA *ra);
TPA_RESULT TpaHL_RA_Serialize(TPA_RA *ra, unsigned char **blob,
		unsigned int *blob_size);
TPA_RESULT TpaHL_RA_Deserialize(unsigned char *blob, unsigned int blob_size,
		unsigned char **blob_end, TPA_RA **ra);

TPA_RESULT TpaHL_RA_Quote(TPA_CONTEXT *context, TPA_TPM *tpm, TPA_AIK *aik,
		TPA_PCR_SET *selectedPcrs, unsigned char *raNonce,
		unsigned int raNonce_size, TPA_RA *raResult);
TPA_RESULT TpaHL_RA_Verify(TPA_CONTEXT *context, TPA_RA *ra, TPA_AIK **aik,
		unsigned char **raNonce, unsigned int *raNonce_size,
		TPA_PCR_SET **pcrs, TPA_MEASURE_SET **measures,
		TPA_VERIFICATION_RESULT *res);

/*@}*/

#endif
