/* ==========================================================================
 * This file is part of the Trusted Platform Agent (TPA)
 *
 * Copyright (C) 2007-2010 Politecnico di Torino, Italy
 * TORSEC group -- http://security.polito.it
 *
 * Trusted Platform Agent (TPA) is copyrighted free software by Politecnico
 * di Torino (TORSEC group).  You can redistribute it and/or modify it under
 * either the terms of the LGPL (see COPYING.LESSER file), or the GPL (see
 * COPYING file).
 *
 * DISCLAIMER of WARRANTY
 *
 * The following software "Trusted Platform Agent (TPA)" developed for
 * Open_TC project (http://www.opentc.net), whilst this version is not
 * endorsed by Open_TC, is experimental and is provided "as is", and no
 * guarantee or warranty is given by Politecnico di Torino (TORSEC group),
 * that the software is fit for any particular purpose.
 * The user thereof uses the software at its sole risk and liability.
 * Politecnico di Torino (TORSEC group) shall have no obligation to maintain
 * or support this software.  Politecnico di Torino (TORSEC group) MAKES NO
 * EXPRESS OR IMPLIED WARRANTY OF ANY KIND REGARDING THIS SOFTWARE.
 * Politecnico di Torino SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED ON
 * CONTRACT, TORT OR ANY OTHER LEGAL THEORY, IN CONNECTION WITH OR
 * ARISING OUT OF THE FURNISHING, PERFORMANCE OR USE OF THIS SOFTWARE.
 * ==========================================================================
 */
#ifndef TPA_COMMON_H_
#define TPA_COMMON_H_

#include "config.h"
#include <stdio.h>
#include <sqlite3.h>
/*********************************************************************/
/* DOXYGEN                                                           */
/*********************************************************************/

/**
 *  @defgroup hlApi High-level API
 *  The High Level API provides access to the underlying functions
 */

/** @defgroup hlApi_tpa TPA Internals
 *  @ingroup hlApi
 *
 */

/** @defgroup hlApi_tpa_ctx Context (CTX)
 *  @ingroup hlApi_tpa
 *
 */

/** @defgroup hlApi_tpa_tpm Trusted Platform Module (TPM)
 *  @ingroup hlApi_tpa
 *
 */

/** @defgroup hlApi_tpa_mod Modules (KEY, MEASURE)
 *  @ingroup hlApi_tpa
 *
 */

/** @defgroup hlApi_tpa_ls Local Storage (LS)
 *  @ingroup hlApi_tpa
 *
 */

/** @defgroup hlApi_key Key Management
 *  @ingroup hlApi
 *
 */

/** @defgroup hlApi_key_ek Endorsement Key (EK)
 *  @ingroup hlApi_key
 *
 */

/** @defgroup hlApi_key_smk Sealing Master Key (SMK)
 *  @ingroup hlApi_key
 *
 */

/** @defgroup hlApi_key_gen Generic TPM Key (KEY, TPMKEY)
 *  @ingroup hlApi_key
 *
 */

/** @defgroup hlApi_key_pca Privacy Certificate Authority (PCA)
 *  @ingroup hlApi_key
 *
 */

/** @defgroup hlApi_key_aik Attestation Identity Key (AIK)
 *  @ingroup hlApi_key
 *
 */

/** @defgroup hlApi_data Data Trust
 *  @ingroup hlApi
 *
 */

/** @defgroup hlApi_data_blob Generic Blob (BLOB)
 *  @ingroup hlApi_data
 *
 */

/** @defgroup hlApi_data_pcr Platform Configuration Register (PCR, PCR_SET)
 *  @ingroup hlApi_data
 *
 */

/** @defgroup hlApi_data_seal Binding, Sealing
 *  @ingroup hlApi_data
 *
 */

/** @defgroup hlApi_rem Remote Reporting
 *  @ingroup hlApi
 *
 */

/** @defgroup hlApi_rem_meas Measure logs (MEASURE, MEASURE_SET)
 *  @ingroup hlApi_rem
 *
 */

/** @defgroup hlApi_rem_ra Remote Attestation (RA)
 *  @ingroup hlApi_rem
 *
 */

/*********************************************************************/
/* DATA TYPES                                                        */
/*********************************************************************/

/* return values */
typedef int TPA_RESULT;
#define TPA_SUCCESS 				0
#define TPA_ERROR 					1
#define TPA_NOT_IMPLEMENTED			2

typedef int TPA_VERIFICATION_RESULT;
#define TPA_VERIFICATION_OK 		TPA_SUCCESS
#define TPA_VERIFICATION_NOK 		!TPA_VERIFICATION_OK

typedef int TPA_OBJ_ID;
typedef int TPA_OBJ_TYPE;
typedef void *TPA_OBJ_PTR;

/* TRACE messages */
#define TPA_PROGRAM_NAME 			PACKAGE_STRING
#define TPA_ERR_LEVEL_INFO			"info"
#define TPA_ERR_LEVEL_WARNING		"warning"
#define TPA_ERR_LEVEL_ERROR			"error"
#define TPA_ERR_LEVEL_BUG			"bug"

#ifdef TRACE
#define TRACE_MSG_INFO(message) fprintf (stderr, "(%s) info - %s:%u %s - %s\n", 		\
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message);
#define TRACE_MSG_WARNING(message) fprintf (stderr, "(%s) warning - %s:%u %s - %s\n", 	\
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message);
#define TRACE_MSG_ERROR(message) fprintf (stderr, "(%s) error - %s:%u %s - %s\n", 		\
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message)
#define TRACE_MSG_BUG(message) fprintf (stderr, "(%s) bug - %s:%u %s - %s\n", 			\
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message);
/* Specific TRACE */
#define TRACE_MSG_INFO_TAG(message,tag) fprintf (stderr, "(%s) info - %s:%u %s - %s (cmd: 0x%x)",	\
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message, tag);
#define TRACE_MSG_INFO_MSG(message,msg) fprintf (stderr, "(%s) info - %s:%u %s - %s (msg: 0x%x)",	\
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message, msg);
#define TRACE_MSG_INFO_INT(message,val) fprintf (stderr, "(%s) info - %s:%u %s - %s (value: %d)",	\
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message, val);
#define TRACE_MSG_INFO_STR(message,str) fprintf (stderr, "(%s) info - %s:%u %s - %s %s",	\
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message, str);
#define TRACE_MSG_TSS_ERROR(message, result) fprintf (stderr, "(%s) error - %s:%u %s - %s - %x:%s\n", 		\
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message, result, Trspi_Error_String(result));
#define TRACE_MSG_PARAM_ERROR(message) fprintf (stderr, "(%s) error - %s:%u %s - Param: %s\n", 		\
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message);
#define TRACE_MSG_ERROR_STR(message,str) fprintf (stderr, "(%s) error - %s:%u %s - %s %s\n",	 \
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message, str);
#define TRACE_MSG_ERROR_MSG(message,msg) fprintf (stderr, "(%s) error - %s:%u %s - %s (msg: %s)\n",	 \
										  TPA_PROGRAM_NAME, __FILE__, 					\
										  __LINE__, __FUNCTION__, 						\
										  message, msg);
#else
#define TRACE_MSG_INFO(message)
#define TRACE_MSG_WARNING(message)
#define TRACE_MSG_ERROR(message)
#define TRACE_MSG_BUG(message)
#define TRACE_MSG_INFO_TAG(message,tag)
#define TRACE_MSG_INFO_MSG(message,msg)
#define TRACE_MSG_INFO_INT(message,val)
#define TRACE_MSG_INFO_STR(message,str)
#define TRACE_MSG_TSS_ERROR(message, result)
#define TRACE_MSG_PARAM_ERROR(message)
#define TRACE_MSG_ERROR_MSG(message,msg)
#endif


/**
 * @addtogroup hlApi_tpa_tpm
 */
/*@{*/

typedef int TPA_TPM_OWNERSHIP_RESULT;
#define TPM_IS_OWNED				1
#define TPM_IS_NOT_OWNED			!TPM_IS_OWNED

typedef struct tpa_tpm {
	char *ownSecret;
	unsigned int ownSecret_len;
	char *srkSecret;
	unsigned int srkSecret_len;
	char *ekCertificateFilename;
	unsigned int ekCertificateFilename_len;
	unsigned char *ekCertificate;
	unsigned int ekCertificate_size;
} TPA_TPM;

/*@}*/

/**
 * @addtogroup hlApi_tpa_ls
 */
/*@{*/

typedef int TPA_LS_TYPE;

typedef struct tpa_ls_drv TPA_LS_DRV;

typedef struct tpa_ls {
	/* DB */
	TPA_LS_TYPE type;
	char *path;
	char *filename;
	char *complete_path;

	/* Driver */
	char *driver_lib;
	TPA_LS_DRV *objp;
	void *dlhandle;
	void *private_data;

    /* FIXME */
    sqlite3 *sqldb;
    char *uri;
} TPA_LS;

/*@}*/

/**
 * @addtogroup hlApi_data_pcr
 */
/*@{*/

#define TPA_PCR_SIZE 				20

typedef int TPA_PCR_NUM;
#define TPA_PCR_NUM_MIN				0
#define TPA_PCR_NUM_MAX				23
#define TPA_PCR_NUM_MAX11			15

typedef struct tpa_pcr {
	TPA_PCR_NUM pcr;
	unsigned char *value;
	unsigned int pcr_size;
} TPA_PCR;

typedef struct {
	unsigned int pcrList_size;
	TPA_PCR *pcrList;
	unsigned int pcr_size;
	int index;
} TPA_PCR_SET;

typedef struct {
	unsigned short int sizeOfSelect;
	unsigned char *pcrSelect;
	unsigned char *digest;
	unsigned int digest_size;
} TPA_PCR_INFO;

/*@}*/

/**
 * @addtogroup hlApi_key
 */
/*@{*/

typedef struct {
	/* TPA_GENERIC_OBJPTR */
	TPA_OBJ_TYPE type;
	TPA_OBJ_PTR objptr;
	TPA_OBJ_ID saved_objid; /* [OUT] */

	/* TPA_GENERIC_TPMKEY */
	TPA_OBJ_ID tpmkey_id;
	TPA_OBJ_ID parent_id;
	unsigned char *blob;
	unsigned int blob_size;
} TPA_GENERIC_OBJPTR, TPA_GENERIC_TPMKEY, TPA_KEY;

typedef struct tpa_tpmkey {
	TPA_OBJ_ID tpmkey_id;
	char *keySecret;
	unsigned int keySecret_len;
	char *label;
	unsigned int label_len;
	TPA_OBJ_TYPE keyType;
	unsigned int keyFlags;
	unsigned char *cert;
	unsigned int cert_size;
	unsigned char *pubKey;
	unsigned int pubKey_size;
	unsigned char *privKey;
	unsigned int privKey_size;
	unsigned char *blob;
	unsigned int blob_size;
	struct tpa_tpmkey *parentKey;
	struct tpa_tpmkey *childKey;
	TPA_OBJ_ID parent_id;
	TPA_PCR_SET *sealingPcrs;
	void *tss_info;
//	unsigned char keySpecificData[20];
} TPA_TPMKEY, TPA_KEY_TPMKEY;

#define TPA_KEY_TYPE_SMK			 1
#define TPA_KEY_TYPE_BINDING	 	 2
#define TPA_KEY_TYPE_SEALING	 	 3
#define TPA_KEY_TYPE_STORAGE	 	 4
#define TPA_KEY_TYPE_SIGNING	 	 5
#define TPA_KEY_TYPE_ATTESTATION	 6

/*@}*/

/**
 * @addtogroup hlApi_key_pca
 */
/*@{*/

typedef int TPA_PCA_TYPE;
typedef int TPA_PCA_CERT_FORMAT;

typedef struct tpa_certificate_item {
	struct tpa_certificate_item *next;
	unsigned char *certificate;
	unsigned int certificate_size;
} TPA_CERTIFICATE_ITEM;

typedef struct tpa_pca {
	TPA_PCA_TYPE type;
	char *uri;
	unsigned int uri_len;
	int port;
//	TPA_PCA_AUTH auth;
	char *pcaCertificateFilename;
	unsigned char *pcaCertificate;
	unsigned int pcaCertificate_size;
	TPA_PCA_CERT_FORMAT pcaCertificate_format;
} TPA_PCA;

/*@}*/

/**
 * @addtogroup hlApi_key_aik
 */
/*@{*/

typedef struct tpa_aik {
	TPA_OBJ_ID aik_id;
	char *aikSecret;
	unsigned int aikSecret_len;
	char *label;
	unsigned int label_len;
	char *labelUTF16;
	unsigned int labelUTF16_len;
	char *uuid;
	unsigned char *cert;
	unsigned int cert_size;
	unsigned char *pubKey;
	unsigned int pubKey_size;
	unsigned char *privKey;
	unsigned int privKey_size;
	unsigned char *keyBlob;
	unsigned int keyBlob_size;
} TPA_AIK;

/*@}*/

/**
 * @addtogroup hlApi_rem_meas
 */
/*@{*/

typedef int TPA_MEASURE_TYPE;
#define TPA_MEASURE_TYPE_TPA		0x00010000
#define TPA_MEASURE_TYPE_GENERIC	0x0
#define TPA_MEASURE_TYPE_FILE		0x1
#define TPA_MEASURE_TYPE_LIBRARY	0x2

typedef int TPA_TIME;

typedef struct {
	TPA_MEASURE_TYPE type;
	unsigned char *value;
	unsigned int value_size;
	char *label;
	unsigned int label_len;
	TPA_PCR *pcrBefore;
	TPA_TIME timestamp;
} TPA_MEASURE;

typedef struct {
	unsigned int measureList_size;
	TPA_MEASURE *measureList;
	int index;
} TPA_MEASURE_SET;

/*@}*/

/**
 * @addtogroup hlApi_rem_ra
 */
/*@{*/

typedef struct {
	unsigned char *raNonce;
	unsigned int raNonce_size;
	TPA_AIK *aik;
	TPA_PCR_SET *pcrs;
	unsigned char *raSign;
	unsigned int raSign_size;
	TPA_MEASURE_SET *measures;
	unsigned int flags;
} TPA_RA;

/* flags related to computation and/or serialization
 * PCRS:     include TPA_PCR_SET into serialized blob
 * MEASURES: query the DB during Quote
 *           include logs into serialized blob (if available)
 * AIK:      include AIK into serialized blob
 * AIK_CERT: include AIK cert into serialized blob
 * DEFAULT:  all (the implemented)
 * PRIVACYCA_COM: compat with PrivacyCA.com = PCRS only
 */
#define TPA_RA_FLAG_NONCE				0x01
#define TPA_RA_FLAG_MEASURES			0x02
#define TPA_RA_FLAG_AIK_PUB				0x04
#define TPA_RA_FLAG_AIK_CERT			0x08
#define TPA_RA_FLAG_AIK_CHAIN			0x10
#define TPA_RA_FLAG_DEFAULT				TPA_RA_FLAG_NONCE | TPA_RA_FLAG_AIK_CERT | TPA_RA_FLAG_AIK_CHAIN
#define TPA_RA_FLAG_PRIVACYCA_COM		0

#define TPA_RA_SERIAL_NONCE			0x01
#define TPA_RA_SERIAL_MEASURES		0x02
#define TPA_RA_SERIAL_AIK_PUB		0x03
#define TPA_RA_SERIAL_AIK_CERT		0x04
#define TPA_RA_SERIAL_AIK_PCA		0x05

/*@}*/

/**
 * @addtogroup hlApi_tpa_mod
 */
/*@{*/

typedef struct tpa_module_measure {
	TPA_LS *ls;
	TPA_PCR *targetPcr;
	TPA_RESULT (*preprocessing_callback)(struct tpa_module_measure *, unsigned char *, unsigned int);
	TPA_RESULT (*postprocessing_callback)(struct tpa_module_measure *, char *);
	void *ctx_data;
} TPA_MODULE_MEASURE;

typedef struct {
	TPA_LS *ls;
} TPA_MODULE_KEY;

/*@}*/

#endif
