/* ==========================================================================
 * This file is part of the Trusted Platform Agent (TPA)
 *
 * Copyright (C) 2007-2010 Politecnico di Torino, Italy
 * TORSEC group -- http://security.polito.it
 *
 * Trusted Platform Agent (TPA) is copyrighted free software by Politecnico
 * di Torino (TORSEC group).  You can redistribute it and/or modify it under
 * either the terms of the LGPL (see COPYING.LESSER file), or the GPL (see
 * COPYING file).
 *
 * DISCLAIMER of WARRANTY
 *
 * The following software "Trusted Platform Agent (TPA)" developed for
 * Open_TC project (http://www.opentc.net), whilst this version is not
 * endorsed by Open_TC, is experimental and is provided "as is", and no
 * guarantee or warranty is given by Politecnico di Torino (TORSEC group),
 * that the software is fit for any particular purpose.
 * The user thereof uses the software at its sole risk and liability.
 * Politecnico di Torino (TORSEC group) shall have no obligation to maintain
 * or support this software.  Politecnico di Torino (TORSEC group) MAKES NO
 * EXPRESS OR IMPLIED WARRANTY OF ANY KIND REGARDING THIS SOFTWARE.
 * Politecnico di Torino SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED ON
 * CONTRACT, TORT OR ANY OTHER LEGAL THEORY, IN CONNECTION WITH OR
 * ARISING OUT OF THE FURNISHING, PERFORMANCE OR USE OF THIS SOFTWARE.
 * ==========================================================================
 */
#ifndef TPA_LS_H_
#define TPA_LS_H_

#include "TPA_Common.h"
#include "TPA_LS_Driver.h"

#define DB_OBJECT_NOT_FOUND                                      -1

#define DB_OBJECT_TYPE_SMK                                        0
#define DB_OBJECT_TYPE_AIK                                        1
#define DB_OBJECT_TYPE_SKAE                                       2
#define DB_OBJECT_TYPE_BK                                         3
#define DB_OBJECT_TYPE_KCERT                                      4
#define DB_OBJECT_TYPE_BLOB                                       5

TPA_RESULT Tpa_LS_allocate(TPA_LS **ls);
void Tpa_LS_freeMemory(TPA_LS *ls);

TPA_RESULT Tpa_LS_Init(TPA_LS *ls);
TPA_RESULT Tpa_LS_Open(TPA_LS *ls);
TPA_RESULT Tpa_LS_Close(TPA_LS *ls);
TPA_RESULT Tpa_LS_KeyLoad(TPA_LS *ls, TPA_KEY *key);
TPA_RESULT Tpa_LS_KeyStore(TPA_LS *ls, TPA_KEY *key);
TPA_RESULT Tpa_LS_KeyDelete(TPA_LS *ls, TPA_KEY *key);
TPA_RESULT Tpa_LS_KeyList(TPA_LS *ls, TPA_KEY *key);
TPA_RESULT Tpa_LS_KeyIsPresent(TPA_LS *ls, TPA_KEY *key);
TPA_RESULT Tpa_LS_MeasureStore(TPA_LS *ls, TPA_MEASURE *measure);
TPA_RESULT Tpa_LS_MeasureDeleteAll(TPA_LS *ls);
TPA_RESULT Tpa_LS_MeasureSetList(TPA_LS *ls, TPA_MEASURE_SET *measure_set);
TPA_RESULT Tpa_LS_MeasureSetDelete(TPA_LS *ls, TPA_MEASURE_SET *measure_set);

/*
 * OLD Stuffs
 * FIXME: remove me!
 */
#include "TPA_API.h"
TPA_RESULT TpaHL_LS_init(TPA_CONTEXT *context);
TPA_RESULT LS_Init(TPA_LS *);
TPA_RESULT LS_Init_new(TPA_CONTEXT *, TPA_LS **);
void LS_FreeMemory(TPA_LS *ls);

#define DB_AIK_TYPE                                0
#define DB_STORAGEMK_TYPE                3
#define DB_BLOB_TYPE                        4
#define DB_BINDINGKEY_TYPE                5

#define DB_KEY_NOT_FOUND                -1

TPA_RESULT TpaLS_AIK_Store(TPA_CONTEXT *, TPA_AIK *, TPA_LS *);
TPA_RESULT TpaLS_AIK_Retrieve(TPA_CONTEXT *, TPA_AIK *, TPA_LS *);
TPA_RESULT TpaLS_AIK_Delete(TPA_CONTEXT *, TPA_AIK *, TPA_LS *);
TPA_RESULT
                TpaLS_AIK_List(TPA_CONTEXT *, TPA_AIK **, int *, TPA_LS *);
TPA_RESULT TpaLS_AIK_HD_Store(TPA_CONTEXT *, TPA_AIK *, TPA_LS *);
TPA_RESULT TpaLS_AIK_HD_Retrieve(TPA_CONTEXT *, TPA_AIK *, TPA_LS *);
TPA_RESULT TpaLS_AIK_HD_Delete(TPA_CONTEXT *, TPA_AIK *, TPA_LS *);
TPA_RESULT TpaLS_AIK_HD_List(TPA_CONTEXT *, TPA_AIK **, int *,
                TPA_LS *);
TPA_RESULT TpaLS_AIK_DB_Store(TPA_CONTEXT *, TPA_AIK *, TPA_LS *);
TPA_RESULT TpaLS_AIK_DB_Retrieve(TPA_CONTEXT *, TPA_AIK *, TPA_LS *);
TPA_RESULT TpaLS_AIK_DB_Delete(TPA_CONTEXT *, TPA_AIK *, TPA_LS *);
TPA_RESULT TpaLS_AIK_DB_List(TPA_CONTEXT *, TPA_AIK **, int *,
                TPA_LS *);
TPA_RESULT TpaLS_Cert_DB_Store(TPA_CONTEXT *, TPA_CERTIFICATE_ITEM *,
                TPA_LS *);
TPA_RESULT TpaLS_Cert_HD_Store(TPA_CONTEXT *, TPA_CERTIFICATE_ITEM *,
                TPA_LS *);
TPA_RESULT TpaLS_DB_Open(TPA_CONTEXT *context, TPA_LS *ls);
TPA_RESULT TpaLS_DB_Close(TPA_CONTEXT *context, TPA_LS *ls);
TPA_RESULT TpaLS_DB_Init(TPA_LS *ls);
TPA_RESULT TpaLS_DB_Store(TPA_CONTEXT *context, TPA_GENERIC_OBJPTR *key,
                TPA_LS *ls);
TPA_RESULT TpaLS_DB_LoadById(TPA_CONTEXT *context, TPA_GENERIC_OBJPTR *obj,
                TPA_LS *ls);
TPA_RESULT TpaLS_DB_DeleteById(TPA_CONTEXT *context, TPA_OBJ_ID key_id,
                TPA_OBJ_TYPE type, TPA_LS *ls);
TPA_RESULT TpaLS_DB_LoadStorageMK(TPA_CONTEXT *context,
                TPA_GENERIC_OBJPTR *obj, TPA_LS *ls);
TPA_RESULT TpaLS_DB_LookForStorageMK(TPA_CONTEXT *context,
                TPA_GENERIC_OBJPTR *obj, TPA_LS *ls);
TPA_RESULT TpaLS_DB_List(TPA_CONTEXT *context, TPA_LS *ls,
                TPA_OBJ_TYPE kType, int **key_list, int *key_list_size);
TPA_RESULT TpaLS_LS_allocate(TPA_LS **ls);
void TpaLS_LS_freeMemory(TPA_LS *ls);
FILE * TpaLS_LS_Open(TPA_LS *, char *);
FILE * TpaLS_LS_Open2(TPA_LS *, char *, char *);
int TpaLS_LS_Len(FILE *);
char* TpaLS_LS_AbsolutePath(TPA_LS *, char *);


#endif /* TPA_LS_H_ */
