/* ==========================================================================
 * This file is part of the Trusted Platform Agent (TPA)
 *
 * Copyright (C) 2007-2010 Politecnico di Torino, Italy
 * TORSEC group -- http://security.polito.it
 *
 * Trusted Platform Agent (TPA) is copyrighted free software by Politecnico
 * di Torino (TORSEC group).  You can redistribute it and/or modify it under
 * either the terms of the LGPL (see COPYING.LESSER file), or the GPL (see
 * COPYING file).
 *
 * DISCLAIMER of WARRANTY
 *
 * The following software "Trusted Platform Agent (TPA)" developed for
 * Open_TC project (http://www.opentc.net), whilst this version is not
 * endorsed by Open_TC, is experimental and is provided "as is", and no
 * guarantee or warranty is given by Politecnico di Torino (TORSEC group),
 * that the software is fit for any particular purpose.
 * The user thereof uses the software at its sole risk and liability.
 * Politecnico di Torino (TORSEC group) shall have no obligation to maintain
 * or support this software.  Politecnico di Torino (TORSEC group) MAKES NO
 * EXPRESS OR IMPLIED WARRANTY OF ANY KIND REGARDING THIS SOFTWARE.
 * Politecnico di Torino SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED ON
 * CONTRACT, TORT OR ANY OTHER LEGAL THEORY, IN CONNECTION WITH OR
 * ARISING OUT OF THE FURNISHING, PERFORMANCE OR USE OF THIS SOFTWARE.
 * ==========================================================================
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "TPA_PCA.h"

#include <curl/curl.h>

#define	EKSIZE		256 /*(2048/8)*/

/*
 * from Hal Finney code
 * */
TPA_RESULT Tpa_PCA_Helper_AIKRequest(TPA_PCA *ca, char *request,
		int request_size, char **asymBlob_bin, int *asymBlob_bin_size,
		char **symBlob_bin, int *symBlob_bin_size) {

	CURL *hCurl;
	FILE *f_tmp = NULL;
	const int level = 1;
	char url[128];
	struct curl_slist *slist = NULL;
	unsigned char *response = NULL;
	int response_size = 0;
	TPA_RESULT result = TPA_ERROR;

	curl_global_init(CURL_GLOBAL_ALL);

	/* Send to server */
	f_tmp = tmpfile();
	hCurl = curl_easy_init();
#ifndef FIXME
	/* Check ca->uri field */
	sprintf(url, REQURL, level);
#endif
	curl_easy_setopt (hCurl, CURLOPT_URL, url);
	curl_easy_setopt (hCurl, CURLOPT_POSTFIELDS, request);
	curl_easy_setopt (hCurl, CURLOPT_POSTFIELDSIZE, request_size);
	curl_easy_setopt (hCurl, CURLOPT_WRITEDATA, (unsigned char **)f_tmp);
	slist = curl_slist_append(slist, "Pragma: no-cache");
	slist = curl_slist_append(slist, "Content-Type: application/octet-stream");
	slist = curl_slist_append(slist, "Content-Transfer-Encoding: binary");
	curl_easy_setopt (hCurl, CURLOPT_HTTPHEADER, slist);
	if (curl_easy_perform(hCurl)) {
		TRACE_MSG_ERROR(
				"Unable to connect to Privacy CA, curl library error");
		curl_slist_free_all(slist);
		goto error;
	}
	curl_slist_free_all(slist);

	TRACE_MSG_INFO("Processing response...\n");

	fflush(f_tmp);
	response_size = ftell(f_tmp);
	response = malloc(response_size);
	rewind(f_tmp);

	fread(response, 1, response_size, f_tmp);
	fclose(f_tmp);
	f_tmp = NULL;

	*asymBlob_bin_size = EKSIZE;
	if (response_size <= *asymBlob_bin_size) {
		TRACE_MSG_ERROR("Bad response from PrivacyCA.com, invalid response_size");
		goto error;
	}
	*asymBlob_bin = (char *) malloc(*asymBlob_bin_size);
	memcpy(*asymBlob_bin, response, *asymBlob_bin_size);

	*symBlob_bin_size = response_size - *asymBlob_bin_size;
	*symBlob_bin = (char *) malloc(*symBlob_bin_size);
	memcpy(*symBlob_bin, response + *asymBlob_bin_size, *symBlob_bin_size);

	result = TPA_SUCCESS;
	error: curl_global_cleanup();
	if (f_tmp)
		fclose(f_tmp);
	if (response)
		free(response);
	return result;

}

TPA_RESULT Tpa_PCA_Helper_DownloadPCACert(TPA_PCA *ca) {

	CURL *hCurl = NULL;
	char url[128];
	FILE *f_tmp = tmpfile();
	TPA_RESULT result = TPA_ERROR;
	unsigned char *response = NULL;
	const int level = 1;
	int response_size = 0;

	curl_global_init(CURL_GLOBAL_ALL);

	hCurl = curl_easy_init();
	//curl_easy_setopt (hCurl, CURLOPT_VERBOSE, 1);
#ifndef FIXME
	/* Check ca->uri field */
	sprintf(url, CERTURL, level);
#endif
	curl_easy_setopt (hCurl, CURLOPT_URL, url);
	curl_easy_setopt(hCurl, CURLOPT_WRITEDATA, (unsigned char **)f_tmp);

	if (curl_easy_perform(hCurl)) {
		TRACE_MSG_ERROR(
				"Unable to connect to Privacy CA, curl library error");
		goto error;
	}

	fflush(f_tmp);
	response_size = ftell(f_tmp);
	response = malloc(response_size);
	rewind(f_tmp);

	fread(response, 1, response_size, f_tmp);

	ca->pcaCertificate = response;
	ca->pcaCertificate_size = response_size;

	result = TPA_SUCCESS;
	error: curl_global_cleanup();
	if (f_tmp)
		fclose(f_tmp);
	return result;
}

TPA_RESULT Tpa_PCA_Helper_AIKLocate(TPA_PCA *ca, char* label, int label_size,
		TPA_CERTIFICATE_ITEM **cert_list) {
	return TPA_NOT_IMPLEMENTED;
}

TPA_RESULT Tpa_PCA_Helper_Validate(TPA_PCA *ca, unsigned char *cert,
		unsigned int cert_size, TPA_CERTIFICATE_ITEM **cert_list,
		TPA_VERIFICATION_RESULT *res) {
	return TPA_NOT_IMPLEMENTED;
}

TPA_RESULT Tpa_PCA_Helper_Revoke(TPA_PCA *ca, char *cert, int cert_size) {
	return TPA_NOT_IMPLEMENTED;
}

TPA_RESULT Tpa_PCA_Helper_EKCertify(TPA_PCA *ca, unsigned char *modulus,
		unsigned int modulus_size, unsigned char *exponent,
		unsigned int exponent_size, unsigned char **certificate,
		unsigned int *certificate_size) {
	return TPA_NOT_IMPLEMENTED;
}

#ifdef IAIK_PCA

#include "iaik/connection.h"
#include "iaik/pki_parser.h"
#include "iaik/pki_emitter.h"
#include "iaik/parsercore.h"
#include "iaik/pki_constants.h"
#include "iaik/rsa_key.h"
#include "iaik/base64.h"
#include "iaik/cert.h"

extern void parse_pki_response(struct pki_parser* parser,
		struct conn_info* conn);

TPA_RESULT Tpa_PCA_Helper_AIKRequest(TPA_PCA *ca, char *request,
		int request_size, char** asymBlob_bin, int *asymBlob_bin_size,
		char** symBlob_bin, int *symBlob_bin_size) {

	struct conn_info* conn = NULL;
	struct pki_parser* pkiparser = NULL;
	struct parsehash_item* asymBlob_base64 = NULL;
	struct parsehash_item* symBlob_base64 = NULL;

	conn = calloc(1, sizeof(struct conn_info));

	if (open_connection(conn, ca->uri, ca->port) == NULL) {
		TRACE_MSG_ERROR("open_connection failed");
		return TPA_ERROR;
	}

	pkiparser = calloc(1, sizeof(struct pki_parser));

	pki_parser_init(pkiparser, 4096);
	emit_create_aik_request(conn, request, request_size);
	pkiparser->last_cmd = APKI_CREATE_AIK_REQUEST;
	free(request);

	parse_pki_response(pkiparser, conn);
	close_connection(conn);

	asymBlob_base64 = hash_find(pkiparser->headers, (char*) APKI_BLOBASYM);
	symBlob_base64 = hash_find(pkiparser->headers, (char*) APKI_BLOBSYM);

	if ((pkiparser->cmd_finished == true) && (asymBlob_base64 != NULL)) {
		*asymBlob_bin = base64_to_bin((asymBlob_base64->value), strlen(
						asymBlob_base64->value), asymBlob_bin_size);
		*symBlob_bin = base64_to_bin(symBlob_base64->value, strlen(
						symBlob_base64->value), symBlob_bin_size);

		if ((!*asymBlob_bin) || (!*symBlob_bin)) {
			printf("base64 decode of response blobs failed!\n");
			return TPA_ERROR;
		}
		return TPA_SUCCESS;
	}

	return TPA_ERROR;
}

TPA_RESULT Tpa_PCA_Helper_AIKLocate(TPA_PCA *ca, char* label, int label_len,
		TPA_CERTIFICATE_ITEM **cert_list) {

	struct conn_info* conn = NULL;
	struct pki_parser* pkiparser = NULL;

	if (*cert_list != NULL) {
		return TPA_ERROR;
	}

	conn = calloc(1, sizeof(struct conn_info));

	if (open_connection(conn, ca->uri, ca->port) == NULL) {
		TRACE_MSG_ERROR("open_connection failed");
		return TPA_ERROR;
	}

	pkiparser = calloc(1, sizeof(struct pki_parser));
	pki_parser_init(pkiparser, 4096);

	emit_locate_aik_request(conn, label, label_len);
	pkiparser->last_cmd = APKI_LOCATE_AIK_REQUEST;

	parse_pki_response(pkiparser, conn);
	close_connection(conn);

	if (pkiparser->cmd_finished == true) {

		if (pkiparser->certificates != NULL && pkiparser->certificates->data
				==NULL) {
			//&& (pkiparser->certificates == NULL)) {
			printf("no match found\n");
		} else {
			*cert_list = (TPA_CERTIFICATE_ITEM *) pkiparser->certificates;
			print_dump_certificate_chain(pkiparser->certificates, NULL);
			pkiparser->certificates = NULL;
		}
	}

	free(conn);
	pki_parser_free(pkiparser);
	free(pkiparser);

	cleanup_openssl();

	return TPA_SUCCESS;
}

TPA_RESULT Tpa_PCA_Helper_Validate(TPA_PCA *ca, unsigned char *cert,
		unsigned int cert_size, TPA_CERTIFICATE_ITEM **cert_list,
		TPA_AIK_VERIFICATION_RESULT *res) {

	struct conn_info* conn = NULL;
	struct pki_parser* pkiparser = NULL;
	char *strOk = NULL;

	*res = 1;

	strOk = strdup("VALID. [V]VALIDITY_INTERVAL,[V]ISSUER_TRUST,[V]SIGNATURE");

	conn = calloc(1, sizeof(struct conn_info));

	if (open_connection(conn, ca->uri, ca->port) == NULL) {
		TRACE_MSG_ERROR("open_connection failed");
		return TPA_ERROR;
	}

	pkiparser = calloc(1, sizeof(struct pki_parser));
	pki_parser_init(pkiparser, 4096);

	emit_validate_aik_request(conn, (char *) cert, (int) cert_size);
	pkiparser->last_cmd = APKI_VALIDATE_AIK_REQUEST;
	//free(cert); ??????????????????????

	parse_pki_response(pkiparser, conn);
	close_connection(conn);

	/* TODO: fixme! */

	struct parsehash_item* status = hash_find(pkiparser->headers,
			(char*) APKI_CERTIFICATE_STATUS);

	if ((pkiparser->cmd_finished == true) && (status != NULL)) {
		//printf("%s\n", status->value);
		if (strcmp(strOk, status->value) == 0) {
			*res = 0;
		}

		*cert_list = (TPA_CERTIFICATE_ITEM *) pkiparser->certificates;
		//print_dump_certificate_chain(pkiparser->certificates, NULL);
		pkiparser->certificates = NULL;
	}

	free(conn);
	pki_parser_free(pkiparser);
	free(pkiparser);
	free(strOk);

	cleanup_openssl();

	return TPA_SUCCESS;
}

TPA_RESULT Tpa_PCA_Helper_Revoke(TPA_PCA *ca, char *cert, int cert_size) {

	struct conn_info* conn = NULL;

	conn = calloc(1, sizeof(struct conn_info));

	if (open_connection(conn, ca->uri, ca->port) == NULL) {
		TRACE_MSG_ERROR("open_connection failed");
		return TPA_ERROR;
	}

	struct pki_parser* pkiparser = calloc(1, sizeof(struct pki_parser));
	pki_parser_init(pkiparser, 4096);

	emit_revoke_aik_request(conn, cert, cert_size);
	pkiparser->last_cmd = APKI_REVOKE_AIK_REQUEST;
	//// free(cert); ????????????????????????????

	parse_pki_response(pkiparser, conn);
	close_connection(conn);

	struct parsehash_item* status = hash_find(pkiparser->headers,
			(char*) APKI_CERTIFICATE_STATUS);

	if ((pkiparser->cmd_finished == true) && (status != NULL)) {
		printf("%s\n", status->value);
	}

	free(conn);
	pki_parser_free(pkiparser);
	free(pkiparser);

	/* fix me */
	return TPA_SUCCESS;

	cleanup_openssl();
}

TPA_RESULT Tpa_PCA_Helper_EKCertify(TPA_PCA *ca, unsigned char *modulus,
		unsigned int modulus_size, unsigned char *exponent,
		unsigned int exponent_size, unsigned char **certificate,
		unsigned int *certificate_size) {

	struct conn_info* conn = NULL;
	struct pki_parser* pkiparser = NULL;

	struct rsa_key key = {0};
	key.public_exponent = (char *)exponent;
	key.public_exponent_size = exponent_size;
	key.modulus = (char *) modulus;
	key.modulus_size = modulus_size;

	conn = calloc(1, sizeof(struct conn_info));

	if (open_connection(conn, ca->uri, ca->port) == NULL) {
		TRACE_MSG_ERROR("open_connection failed");
		return TPA_ERROR;
	}

	pkiparser = calloc(1, sizeof(struct pki_parser));

	pki_parser_init(pkiparser, 4096);

	// EK CREATE
	emit_create_ek_request(conn, &key);
	pkiparser->last_cmd = APKI_CREATE_EK_REQUEST;

	parse_pki_response(pkiparser, conn);
	close_connection(conn);

	if ((pkiparser->cmd_finished == true) && (pkiparser->certificates != NULL)) {

		if (pkiparser->certificates->data == NULL
				|| pkiparser->certificates->size == 0) {
			*certificate = NULL;
			*certificate_size = 0;
			return TPA_ERROR;
		}

		*certificate = malloc(pkiparser->certificates->size);
		memcpy(*certificate, pkiparser->certificates->data,
				pkiparser->certificates->size);
		*certificate_size = pkiparser->certificates->size;
	}

	free(conn);
	pki_parser_free(pkiparser);
	free(pkiparser);

	cleanup_openssl();
	//

	return TPA_SUCCESS;
}
#endif

