/* ==========================================================================
 * This file is part of the Trusted Platform Agent (TPA)
 *
 * Copyright (C) 2007-2010 Politecnico di Torino, Italy
 * TORSEC group -- http://security.polito.it
 *
 * Trusted Platform Agent (TPA) is copyrighted free software by Politecnico
 * di Torino (TORSEC group).  You can redistribute it and/or modify it under
 * either the terms of the LGPL (see COPYING.LESSER file), or the GPL (see
 * COPYING file).
 *
 * DISCLAIMER of WARRANTY
 *
 * The following software "Trusted Platform Agent (TPA)" developed for
 * Open_TC project (http://www.opentc.net), whilst this version is not
 * endorsed by Open_TC, is experimental and is provided "as is", and no
 * guarantee or warranty is given by Politecnico di Torino (TORSEC group),
 * that the software is fit for any particular purpose.
 * The user thereof uses the software at its sole risk and liability.
 * Politecnico di Torino (TORSEC group) shall have no obligation to maintain
 * or support this software.  Politecnico di Torino (TORSEC group) MAKES NO
 * EXPRESS OR IMPLIED WARRANTY OF ANY KIND REGARDING THIS SOFTWARE.
 * Politecnico di Torino SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED ON
 * CONTRACT, TORT OR ANY OTHER LEGAL THEORY, IN CONNECTION WITH OR
 * ARISING OUT OF THE FURNISHING, PERFORMANCE OR USE OF THIS SOFTWARE.
 * ==========================================================================
 */
#ifndef TPA_TSS_H_
#define TPA_TSS_H_

#include "TPA_Utils.h"
#include <trousers/tss.h>
#include <trousers/trousers.h>

#define BSIZE								   128
#define TCG_TAG_PCCLIENT_STORED_CERT		0x1001
#define TCG_TAG_PCCLIENT_FULL_CERT			0x1002
#define TCG_TAG_PCCLIENT_PART_SMALL_CERT	0x1003
#define TCG_FULL_CERT							 0
#define TCG_PARTIAL_SMALL_CERT					 1

#define TPA_KEY_TSSFLAGS_AIK	 	(TSS_KEY_TYPE_IDENTITY | TSS_KEY_SIZE_2048 | \
									 TSS_KEY_VOLATILE | TSS_KEY_AUTHORIZATION  | \
									 TSS_KEY_NOT_MIGRATABLE)
#define TPA_KEY_TSSFLAGS_STORAGE	(TSS_KEY_TYPE_STORAGE | TSS_KEY_SIZE_2048  | \
									 TSS_KEY_VOLATILE | TSS_KEY_AUTHORIZATION  | \
									 TSS_KEY_NOT_MIGRATABLE)
#define TPA_KEY_TSSFLAGS_BINDING	(TSS_KEY_TYPE_BIND | TSS_KEY_SIZE_2048  | \
									 TSS_KEY_VOLATILE | TSS_KEY_AUTHORIZATION  | \
									 TSS_KEY_NOT_MIGRATABLE)
#define TPA_KEY_TSSFLAGS_SIGNATURE (TSS_KEY_TYPE_SIGNING | TSS_KEY_SIZE_2048  | \
									 TSS_KEY_VOLATILE | TSS_KEY_AUTHORIZATION  | \
									 TSS_KEY_NOT_MIGRATABLE)

typedef char * TPA_TSS_KEYPWD;
typedef unsigned char * TPA_TSS_KEYBLOB;

typedef struct {
	TSS_HCONTEXT hContext;
	TSS_HTPM hTPM;
	TSS_HKEY hSRK;
	TSS_HKEY hIdentKey;
	TSS_HKEY hPCAKey;
	TSS_HKEY hPubEndKey;
	TSS_HKEY hStorageMK;
	TSS_HPCRS hPcrs;
	void *data;
	unsigned int data_size;
} TPA_TSS_ACCESS;

typedef struct tssKey {
	TSS_HKEY hKey;
	TSS_HKEY hKeyParent;
	TSS_FLAG keyFlags;
	TPA_TSS_KEYBLOB blob;
	unsigned int blob_size;
	TPA_TSS_KEYPWD secret;
	unsigned int secret_size;
} TPA_TSS_KEY;

TPA_RESULT Tpa_TSS_Access_Allocate(TPA_TSS_ACCESS **tss);
void Tpa_TSS_Cleanup(TPA_TSS_ACCESS *tss);
TPA_RESULT Tpa_TSS_Connect(TPA_TSS_ACCESS *tss);
TPA_RESULT Tpa_TSS_SetOwnerSecret(TPA_TSS_ACCESS *tss, char *secret,
		unsigned int secret_len);
TPA_RESULT Tpa_TSS_LoadSRK(TPA_TSS_ACCESS *tss, char *secret,
		unsigned int secret_len);
TPA_RESULT Tpa_TSS_CreateAIKkey(TPA_TSS_ACCESS *tss, char *secret,
		unsigned int secret_len);
TPA_RESULT Tpa_TSS_LoadAIKkey(TPA_TSS_ACCESS *tss, char *secret,
		unsigned int secret_len, unsigned char *keyBlob,
		unsigned int keyBlob_size);
TPA_RESULT Tpa_TSS_CreatePCAkey(TPA_TSS_ACCESS *tss, TPA_PCA *ca);
TPA_RESULT Tpa_TSS_SetEKcert(TPA_TSS_ACCESS *tss, unsigned char *cert,
		unsigned int cert_size);
TPA_RESULT Tpa_TSS_CollateIdentityRequest(TPA_TSS_ACCESS *tss, char *label,
		unsigned int label_len, unsigned char **request,
		unsigned int *request_size);
TPA_RESULT Tpa_TSS_ActivateIdentity(TPA_TSS_ACCESS *tss,
		unsigned char *asymBlob, unsigned int asymBlob_size,
		unsigned char *symBlob, unsigned int symBlob_size,
		unsigned char **credential, unsigned int *credential_size);
TPA_RESULT Tpa_TSS_GetKeyBlob(TPA_TSS_ACCESS *tss, TSS_HKEY tsskey,
		unsigned char **return_data, unsigned int *blob_size);
TPA_RESULT Tpa_TSS_GetKeyModulus(TPA_TSS_ACCESS *tss, TSS_HKEY hKey,
		unsigned char **blob, unsigned int *blob_size);
TPA_RESULT Tpa_TSS_GetKeyExponent(TPA_TSS_ACCESS *tss, TSS_HKEY hKey,
		unsigned char **blob, unsigned int *blob_size);
TPA_RESULT Tpa_TSS_GetRandom(TPA_TSS_ACCESS *tss, unsigned int blob_size,
		unsigned char **blob);
TPA_RESULT Tpa_TSS_CreateStorageMasterKey(TPA_TSS_ACCESS *tss);
TPA_RESULT Tpa_TSS_LoadStorageMasterKey(TPA_TSS_ACCESS *tss,
		unsigned char *keyBlob, unsigned int keyBlob_size);
TPA_RESULT Tpa_TSS_SealBlobWithMK(TPA_TSS_ACCESS *tss,
		TPA_PCR_SET *selectedPcrs, unsigned char *plainblob,
		unsigned int plainblob_size, unsigned char **sealedblob,
		unsigned int *sealedblob_size);
TPA_RESULT Tpa_TSS_UnsealBlobWithMK(TPA_TSS_ACCESS *tss,
		unsigned char *sealedblob, unsigned int sealedblob_size,
		unsigned char **plainblob, unsigned int *plainblob_size);

void Tpa_TSS_GetPcrInfo(BYTE *cInfo, BYTE **pcrInfo, UINT32 *pcrInfo_size);
void Tpa_TSS_GetPcrBitmap(BYTE *pcrInfo, BYTE **bitmap, UINT32 *bitmap_size);
void Tpa_TSS_GetDigestAtRelease(BYTE *pcrInfo, BYTE **digest,
		UINT32 *digest_size);

TPA_RESULT Tpa_TSS_GetTPMPublicEK(TPA_TSS_ACCESS *tss, unsigned char **modulus,
		unsigned int *modulus_size, unsigned char **exponent,
		unsigned int *exponent_size);
TPA_RESULT Tpa_TSS_ReadPCR(TPA_TSS_ACCESS *tss, int pcr_index,
		unsigned char **buffer, unsigned int *buffer_size);
TPA_RESULT Tpa_TSS_ExtendPCR(TPA_TSS_ACCESS *tss, int pcr_index,
		unsigned char *buffer, unsigned int buffer_size,
		unsigned char **extended, unsigned int *extended_size);

TPA_RESULT Tpa_TSS_ReadEKCert(TPA_TSS_ACCESS *tss, unsigned char **buffer,
		unsigned int *buffer_size);

/*
 * New Functions
 */
TPA_RESULT Tpa_TSS_SetSRKSecret(TPA_TSS_ACCESS *tss, char *secret,
		unsigned int secret_len);
TPA_RESULT Tpa_TSS_IsOwnershipTaken(TPA_TSS_ACCESS *tss,
		TPA_TPM_OWNERSHIP_RESULT *isTaken);
TPA_RESULT Tpa_TSS_CreateSRKObject(TPA_TSS_ACCESS *tss, char *secret,
		unsigned int secret_len);
TPA_RESULT Tpa_TSS_TakeOwnership(TPA_TSS_ACCESS *tss);

void Tpa_TSS_KeyFreeMemory(TPA_TSS_ACCESS *tss, TPA_TSS_KEY *key);
TPA_RESULT Tpa_TSS_CreateBindingKey(TPA_TSS_ACCESS *tss,
		TPA_TSS_KEY *parentKey, TPA_TSS_KEYPWD secret,
		unsigned int secret_size, TPA_TSS_KEY **key);
TPA_RESULT Tpa_TSS_LoadBindingKey(TPA_TSS_ACCESS *tss, TPA_TSS_KEYBLOB keyblob,
		unsigned int keyblob_size, TPA_TSS_KEYPWD secret,
		unsigned int secret_size, TPA_TSS_KEY *parentKey, TPA_TSS_KEY **key);
TPA_RESULT Tpa_TSS_Bind(TPA_TSS_ACCESS *tss, TPA_TSS_KEY *key,
		unsigned char *plainblob, unsigned int plainblob_size,
		unsigned char **boundblob, unsigned int *boundblob_size);
TPA_RESULT Tpa_TSS_Unbind(TPA_TSS_ACCESS *tss, TPA_TSS_KEY *key,
		unsigned char *boundblob, unsigned int boundblob_size,
		unsigned char **plainblob, unsigned int *plainblob_size);

TPA_RESULT
		Tpa_TSS_SetPCRIndexes(TPA_TSS_ACCESS *tss, TPA_PCR_SET *selectedPcrs);
TPA_RESULT Tpa_TSS_SetData(TPA_TSS_ACCESS *tss, void *data,
		unsigned int data_size);
TPA_RESULT Tpa_TSS_SetDataBlob(TPA_TSS_ACCESS *tss, unsigned char *data,
		unsigned int data_size);
TPA_RESULT Tpa_TSS_Quote(TPA_TSS_ACCESS *tss, unsigned char ** dataSigned,
		unsigned int * dataSigned_size, unsigned char ** raSignature,
		unsigned int * raSignature_size);
TPA_RESULT Tpa_TSS_Verify(TPA_TSS_ACCESS *tss);

#endif /* TPA_TPM_COMMON_H_ */
