#include "TPA_SQLITE.h"
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <sqlite3.h>

static char *Tpa_LSSQLITE_DrvStr = "Tpa_LSSQLITE_Driver";

TPA_RESULT Tpa_LS_DriverEntry(TPA_LS_DRV *driver_obj) {
	if (driver_obj == NULL) {
		TRACE_MSG_ERROR("driver_obj is NULL");
		return TPA_ERROR;
	}

	driver_obj->Tpa_LS_name = Tpa_LSSQLITE_DrvStr;
	driver_obj->Tpa_LS_Init = Tpa_LSSQLITE_Init;
	driver_obj->Tpa_LS_Open = Tpa_LSSQLITE_Open;
	driver_obj->Tpa_LS_Close = Tpa_LSSQLITE_Close;
	driver_obj->Tpa_LS_KeyLoad = Tpa_LSSQLITE_KeyLoad;
	driver_obj->Tpa_LS_KeyStore = Tpa_LSSQLITE_KeyStore;
	driver_obj->Tpa_LS_KeyDelete = Tpa_LSSQLITE_KeyDelete;
	driver_obj->Tpa_LS_KeyList = Tpa_LSSQLITE_KeyList;
	driver_obj->Tpa_LS_KeyIsPresent = Tpa_LSSQLITE_KeyIsPresent;
	driver_obj->Tpa_LS_MeasureStore = Tpa_LSSQLITE_MeasureStore;
	driver_obj->Tpa_LS_MeasureDeleteAll = Tpa_LSSQLITE_MeasureDeleteAll;
	driver_obj->Tpa_LS_MeasureSetList = Tpa_LSSQLITE_MeasureSetList;
	driver_obj->Tpa_LS_MeasureSetDelete = Tpa_LSSQLITE_MeasureSetDelete;

	return TPA_SUCCESS;
}

TPA_RESULT Tpa_LSSQLITE_Init(TPA_LS *ls) {
	TPA_RESULT result = TPA_ERROR;
	char complete_path[1024] = { 0 };
	char query[MAX_STATEMENT_DIM] = { 0 };
	char *err = NULL;
	sqlite3 *sqldb = NULL;

	if (ls == NULL) {
		TRACE_MSG_PARAM_ERROR("ls is NULL");
		goto error;
	}
	if (ls->path == NULL) {
		TRACE_MSG_PARAM_ERROR("ls->path is NULL");
		goto error;
	}
	if (ls->filename == NULL) {
		TRACE_MSG_PARAM_ERROR("ls->fname is NULL");
		goto error;
	}
	if (ls->complete_path == NULL) {
		snprintf(complete_path, 1024, "%s/%s", ls->path, ls->filename);
		ls->complete_path = strndup(complete_path, 1024);
	}

	if (mkdir(ls->path, 0777) != 0) {
		if (errno != EEXIST) {
			TRACE_MSG_ERROR("impossible to create directory");
			goto error;
		}
	}

	if (sqlite3_open(ls->complete_path, &sqldb) != SQLITE_OK) {
		TRACE_MSG_ERROR_STR("Can't open database:", ls->complete_path);
		goto error;
	}

	if (ls->objp->Tpa_LS_KeyStore != NULL) {
		snprintf(query, MAX_STATEMENT_DIM,
				"CREATE TABLE keys (id INTEGER PRIMARY KEY, type INTEGER); "
					"INSERT INTO keys VALUES(0, 0);");
		if (sqlite3_exec(sqldb, query, NULL, NULL, &err) != SQLITE_OK) {
			fprintf(stderr, "SQL error: %s\n", err);
			sqlite3_free(err);
			goto error;
		}
	}

	if (ls->objp->Tpa_LS_MeasureStore != NULL) {
		snprintf(query, MAX_STATEMENT_DIM,
				"CREATE TABLE measures (id INTEGER PRIMARY KEY, type INTEGER, "
					"label TEXT, value TEXT, pcrnum INTEGER, pcrval TEXT, timestamp DATE);");
		if (sqlite3_exec(sqldb, query, NULL, NULL, &err) != SQLITE_OK) {
			fprintf(stderr, "SQL error: %s\n", err);
			sqlite3_free(err);
			goto error;
		}
	}

	sqlite3_close(sqldb);

	result = TPA_SUCCESS;
	error: return result;
}

TPA_RESULT Tpa_LSSQLITE_Open(TPA_LS *ls) {
	TPA_RESULT result = TPA_ERROR;
	char complete_path[1024] = { 0 };
	sqlite3 *sqldb = NULL;

	if (ls == NULL) {
		TRACE_MSG_PARAM_ERROR("ls is NULL");
		goto error;
	}
	if (ls->path == NULL) {
		TRACE_MSG_PARAM_ERROR("ls->path is NULL");
		goto error;
	}
	if (ls->filename == NULL) {
		TRACE_MSG_PARAM_ERROR("ls->filename is NULL");
		goto error;
	}
	if (ls->complete_path == NULL) {
		snprintf(complete_path, 1024, "%s/%s", ls->path, ls->filename);
		ls->complete_path = strndup(complete_path, 1024);
	}

	if (sqlite3_open(ls->complete_path, &sqldb) != SQLITE_OK) {
		TRACE_MSG_ERROR_STR("Can't open database:", ls->complete_path);
		goto error;
	}
	ls->private_data = sqldb;

	result = TPA_SUCCESS;
	error: return result;
}

TPA_RESULT Tpa_LSSQLITE_Close(TPA_LS *ls) {
	TPA_RESULT result = TPA_ERROR;
	sqlite3 *sqldb = NULL;

	if (ls->private_data == NULL) {
		TRACE_MSG_ERROR("ls->private data is null");
		goto error;
	}

	sqldb = (sqlite3 *) ls->private_data;

	sqlite3_close(sqldb);
	ls->private_data = NULL;

	result = TPA_SUCCESS;
	error: return result;
}

TPA_RESULT Tpa_LSSQLITE_KeyLoad(TPA_LS *ls, TPA_KEY *key) {
	return TPA_NOT_IMPLEMENTED;
}
TPA_RESULT Tpa_LSSQLITE_KeyStore(TPA_LS *ls, TPA_KEY *key) {
	return TPA_NOT_IMPLEMENTED;
}
TPA_RESULT Tpa_LSSQLITE_KeyDelete(TPA_LS *ls, TPA_KEY *key) {
	return TPA_NOT_IMPLEMENTED;
}
TPA_RESULT Tpa_LSSQLITE_KeyList(TPA_LS *ls, TPA_KEY *key) {
	return TPA_NOT_IMPLEMENTED;
}
TPA_RESULT Tpa_LSSQLITE_KeyIsPresent(TPA_LS *ls, TPA_KEY *key) {
	return TPA_NOT_IMPLEMENTED;
}

static void TPA_LSSQLITE_btoa(unsigned char *in, unsigned int len, char *out,
		unsigned int *out_len) {
	int i = 0;
	int count = 0;
	int total = 0;

	for (i = 0; i < len && *out_len > 1; i++) {
		count = snprintf(out, *out_len, "%02x", in[i]);
		out += count;
		*out_len -= count;
		total += count;
	}

	*out_len = total;
	out[total] = '\0';
}

TPA_RESULT Tpa_LSSQLITE_MeasureStore(TPA_LS *ls, TPA_MEASURE *measure) {

	TPA_RESULT result = TPA_ERROR;
	char statement[MAX_STATEMENT_DIM] = { 0 };
	char label[MAX_STRING_DIM] = { 0 };
	char value[MAX_STRING_DIM] = { 0 };
	unsigned int value_size = 0;
	char pcrvalue[MAX_STRING_DIM] = { 0 };
	unsigned int pcrvalue_size = 0;
	sqlite3 *sqldb = NULL;
	int ressqlite = 0;
	char *err = NULL;

	if (ls == NULL) {
		TRACE_MSG_PARAM_ERROR("ls is NULL");
		goto error;
	}
	if (ls->private_data == NULL) {
		TRACE_MSG_ERROR("ls->private data is null");
		goto error;
	}
	if (measure == NULL) {
		TRACE_MSG_PARAM_ERROR("measure is NULL");
		goto error;
	}

	sqldb = (sqlite3 *) ls->private_data;

	if (measure->label_len > 0)
		memcpy(label, measure->label, measure->label_len);

	value_size = MAX_STRING_DIM;
	TPA_LSSQLITE_btoa(measure->value, measure->value_size, value, &value_size);
	pcrvalue_size = MAX_STRING_DIM;
	TPA_LSSQLITE_btoa(measure->pcrBefore->value, measure->pcrBefore->pcr_size,
			pcrvalue, &pcrvalue_size);

	if (snprintf(statement, MAX_STATEMENT_DIM, "INSERT INTO measures "
		"VALUES(NULL, '%d', '%s', '%s', '%d', '%s', DATETIME('now'));",
			measure->type, label, value, measure->pcrBefore->pcr, pcrvalue)
			>= MAX_STATEMENT_DIM) {
		TRACE_MSG_ERROR("Invalid query");
		goto error;
	}

	ressqlite = sqlite3_exec(sqldb, statement, NULL, NULL, &err);
	if (ressqlite != SQLITE_OK) {
		TRACE_MSG_ERROR("sqlite3_exec failed");
		goto error;
	}

	result = TPA_SUCCESS;
	error: return result;
}

TPA_RESULT Tpa_LSSQLITE_MeasureDeleteAll(TPA_LS *ls) {
	TPA_RESULT result = TPA_ERROR;
	char statement[MAX_STATEMENT_DIM] = { 0 };
	sqlite3 *sqldb = NULL;
	int ressqlite = 0;
	char *err = NULL;

	if (ls == NULL) {
		TRACE_MSG_PARAM_ERROR("ls is NULL");
		goto error;
	}
	if (ls->private_data == NULL) {
		TRACE_MSG_ERROR("ls->private data is null");
		goto error;
	}

	sqldb = (sqlite3 *) ls->private_data;

	if (snprintf(statement, MAX_STATEMENT_DIM, "DELETE FROM measures;")
			>= MAX_STATEMENT_DIM) {
		TRACE_MSG_ERROR("Invalid query");
		goto error;
	}

	ressqlite = sqlite3_exec(sqldb, statement, NULL, NULL, &err);
	if (ressqlite != SQLITE_OK) {
		TRACE_MSG_ERROR("sqlite3_exec failed");
		goto error;
	}

	result = TPA_SUCCESS;
	error: return result;

}
TPA_RESULT Tpa_LSSQLITE_MeasureSetList(TPA_LS *ls, TPA_MEASURE_SET *measure_set) {
	return TPA_NOT_IMPLEMENTED;
}
TPA_RESULT Tpa_LSSQLITE_MeasureSetDelete(TPA_LS *ls,
		TPA_MEASURE_SET *measure_set) {
	return TPA_NOT_IMPLEMENTED;
}
