/* ==========================================================================
 * This file is part of the Trusted Platform Agent (TPA)
 *
 * Copyright (C) 2007-2010 Politecnico di Torino, Italy
 * TORSEC group -- http://security.polito.it
 *
 * Trusted Platform Agent (TPA) is copyrighted free software by Politecnico
 * di Torino (TORSEC group).  You can redistribute it and/or modify it under
 * either the terms of the LGPL (see COPYING.LESSER file), or the GPL (see
 * COPYING file).
 *
 * DISCLAIMER of WARRANTY
 *
 * The following software "Trusted Platform Agent (TPA)" developed for
 * Open_TC project (http://www.opentc.net), whilst this version is not
 * endorsed by Open_TC, is experimental and is provided "as is", and no
 * guarantee or warranty is given by Politecnico di Torino (TORSEC group),
 * that the software is fit for any particular purpose.
 * The user thereof uses the software at its sole risk and liability.
 * Politecnico di Torino (TORSEC group) shall have no obligation to maintain
 * or support this software.  Politecnico di Torino (TORSEC group) MAKES NO
 * EXPRESS OR IMPLIED WARRANTY OF ANY KIND REGARDING THIS SOFTWARE.
 * Politecnico di Torino SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED ON
 * CONTRACT, TORT OR ANY OTHER LEGAL THEORY, IN CONNECTION WITH OR
 * ARISING OUT OF THE FURNISHING, PERFORMANCE OR USE OF THIS SOFTWARE.
 * ==========================================================================
 */
#include "TPA_SQLITE.h"

#if 0

/* Constant parameters */
#define MAX_QUERY_DIM 255
#define MAX_LINE_SIZE 255
#define DB_DEFAULT_FILENAME "/tpa.db"
#define PUBKEY_SUFFIX "_pub.pem"
#define PRIKEY_SUFFIX "_pri.blob"
#define PRIKEYSEALED_SUFFIX "_pri.blob.sealed"
#define CERT_SUFFIX "_cert.der"
#define CINFO_SUFFIX "_cinfo.blob"
#define CINFOSIGN_SUFFIX "_cinfo.sig"

/*
 * Open database
 */
TPA_RESULT TpaLS_DB_Open(TPA_CONTEXT *context, TPA_LS *ls) {

	int result = TPA_ERROR;

	if (context->ls_uri == NULL) {
		TRACE_MSG_ERROR("Local storage URI is NULL");
		goto error;
	}

	if (ls->uri == NULL) {
		ls->uri = strdup(context->ls_uri);
		if (ls->uri == NULL) {
			TRACE_MSG_ERROR("strdup failed");
			goto error;
		}
	}

	if (strcmp(ls->uri, context->ls_uri)) {
		TRACE_MSG_ERROR("Context URI and LocalStorage URI don't match");
		goto error;
	}

	if (ls->fname == NULL && ls->complete_path == NULL) {
		ls->fname = strdup(DB_DEFAULT_FILENAME);
		ls->complete_path = (char *) malloc((strlen(ls->uri)
				+ strlen(ls->fname) + 1) * sizeof(char));
		strncpy(ls->complete_path, ls->uri, (strlen(ls->uri)
				+ strlen(ls->fname)));
		strcat(ls->complete_path, ls->fname);
	}

	if (sqlite3_open(ls->complete_path, &(ls->sqldb)) != SQLITE_OK) {
		TRACE_MSG_ERROR_MSG("Can't open database:", ls->complete_path);
		sqlite3_close(ls->sqldb);
		goto error;
	}

	result = TPA_SUCCESS;

	error: /* Free memory and return error code */

	return result;
}

/*
 * Close database
 */
TPA_RESULT TpaLS_DB_Close(TPA_CONTEXT *context, TPA_LS *ls) {

	int result = TPA_ERROR;

	if (ls->sqldb) {
		sqlite3_close(ls->sqldb);
		ls->sqldb = NULL;
	}

	/* free TPA_LOCAL_STORAGE */
	if (ls != NULL) {
		if (ls->uri) {
			free(ls->uri);
			ls->uri = NULL;
		}

		if (ls->fname) {
			free(ls->fname);
			ls->fname = NULL;
		}

		if (ls->complete_path) {
			free(ls->complete_path);
			ls->complete_path = NULL;
		}
	}

	result = TPA_SUCCESS;

	//error: /* Free memory and return error code */

	return result;
}

/*
 * Init DB
 */
TPA_RESULT TpaLS_DB_Init(TPA_LS *ls) {

	TPA_RESULT result = TPA_ERROR;
	char query[MAX_QUERY_DIM] = { 0 };
	char *err = NULL;

	if (ls == NULL) {
		TRACE_MSG_ERROR("ls cannot be null");
		goto error;
	}

	//	if (ls->complete_path == NULL) {
	//		TRACE_MSG_ERROR("ls->complete_path cannot be null");
	//		goto error;
	//	}

	snprintf(query, MAX_QUERY_DIM,
			"CREATE TABLE keys (id INTEGER PRIMARY KEY, type INTEGER); "
				"INSERT INTO keys VALUES(0, 0);");
	/* Execute query */
	if (sqlite3_exec(ls->sqldb, query, NULL, NULL, &err) != SQLITE_OK) {
		fprintf(stderr, "SQL error: %s\n", err);
		sqlite3_free(err);
		goto error;
	}

	result = TPA_SUCCESS;
	error: /* free */
	return result;
}
/*
 * Store object into database
 */
TPA_RESULT TpaLS_DB_Store(TPA_CONTEXT *context, TPA_GENERIC_OBJPTR *obj,
		TPA_LS *ls) {

	TPA_RESULT result = TPA_ERROR;
	int rows = 0;
	int cols = 0;
	//int i = 0;
	int current_id = 0;
	char **mtx = NULL;
	char **mtx2 = NULL;
	char *err = NULL;
	char query[MAX_QUERY_DIM] = { 0 };
	char filename[MAX_LINE_SIZE] = { 0 };
	char *path_filename = NULL;

	if (obj == NULL || obj->objptr == NULL) {
		TRACE_MSG_ERROR("Key structure is not valid.");
		goto error;
	}

	if (ls->sqldb == NULL) {
		TRACE_MSG_ERROR("Database is not open.");
		goto error;
	}

	/* Select id value */
	if (sqlite3_get_table(ls->sqldb, "SELECT max(id) FROM keys;", &mtx, &rows,
			&cols, &err) != SQLITE_OK) {
		TRACE_MSG_ERROR_MSG("SQL error:", err);
		sqlite3_free(err);
		goto error;
	}

	/* Compute next ID value */
	(rows == 0) ? (current_id = 0) : (current_id = atoi(mtx[1]) + 1);
	obj->saved_objid = current_id;

	/* Store the key into DB */
	switch (obj->type) {

	/****************************************************************************/
	/*																			*/
	/* 								Store AIK									*/
	/*																			*/
	/****************************************************************************/
	case DB_AIK_TYPE:
		((TPA_AIK *) (obj->objptr))->aik_id = current_id;
		/* Update keys table */
		snprintf(query, MAX_QUERY_DIM, "INSERT INTO keys VALUES(%d, %d);",
				current_id, DB_AIK_TYPE);
		/* Execute query */
		if (sqlite3_exec(ls->sqldb, query, NULL, NULL, &err) != SQLITE_OK) {
			fprintf(stderr, "SQL error: %s\n", err);
			sqlite3_free(err);
			goto error;
		}
		/* Save blobs into storage */
		snprintf(filename, MAX_LINE_SIZE, "%d"PUBKEY_SUFFIX, current_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for pubkey failed");
			goto error;
		}
		if (TpaHL_GenTpmKey_writePubKeyToFile(path_filename,
				((TPA_AIK *) (obj->objptr))->pubKey,
				((TPA_AIK *) (obj->objptr))->pubKey_size) != TPA_SUCCESS) {
			TRACE_MSG_ERROR("TpaHL_BLOB_writePubBlobToFile failed");
			goto error;
		}
		free(path_filename);
		path_filename = NULL;

		snprintf(filename, MAX_LINE_SIZE, "%d"PRIKEY_SUFFIX, current_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for privkey failed");
			goto error;
		}
		if (Tpa_BLOB_writeToFile(path_filename,
				((TPA_AIK *) (obj->objptr))->keyBlob,
				((TPA_AIK *) (obj->objptr))->keyBlob_size) != TPA_SUCCESS) {
			TRACE_MSG_ERROR("TpaHL_BLOB_writeBlobToFile (aik blob) failed");
			goto error;
		}
		free(path_filename);
		path_filename = NULL;

		snprintf(filename, MAX_LINE_SIZE, "%d"CERT_SUFFIX, current_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for cert failed");
			goto error;
		}
		if (Tpa_BLOB_writeToFile(path_filename,
				((TPA_AIK *) (obj->objptr))->cert,
				((TPA_AIK *) (obj->objptr))->cert_size) != TPA_SUCCESS) {
			TRACE_MSG_ERROR("TpaHL_BLOB_writeBlobToFile (cert) failed");
			goto error;
		}

		break;

		/****************************************************************************/
		/*																			*/
		/* 						Store Storage Master Key							*/
		/*																			*/
		/****************************************************************************/
	case DB_STORAGEMK_TYPE:
		/* master key must be unique */
		snprintf(query, MAX_QUERY_DIM, "SELECT id FROM keys WHERE type=%d;",
				DB_STORAGEMK_TYPE);
		/* Select id value */
		if (sqlite3_get_table(ls->sqldb, query, &mtx2, &rows, &cols, &err)
				!= SQLITE_OK) {
			TRACE_MSG_ERROR_MSG("SQL error:", err);
			sqlite3_free(err);
			goto error;
		}

		if (rows == 1) {
			TRACE_MSG_ERROR("storage master key already created");
			goto error;
		} else if (rows > 1) {
			TRACE_MSG_ERROR("internal db error");
			goto error;
		} else if (rows == 0) {
			/* Set current SEALMK id */
			((TPA_GENERIC_TPMKEY *) (obj->objptr))->tpmkey_id = current_id;
			/* Update keys table */
			snprintf(query, MAX_QUERY_DIM, "INSERT INTO keys VALUES(%d, %d);",
					current_id, DB_STORAGEMK_TYPE);
			/* Execute query */
			if (sqlite3_exec(ls->sqldb, query, NULL, NULL, &err) != SQLITE_OK) {
				fprintf(stderr, "SQL error: %s\n", err);
				sqlite3_free(err);
				goto error;
			}
			/* Save blob into storage */
			snprintf(filename, MAX_LINE_SIZE, "%d"PRIKEY_SUFFIX, current_id);
			path_filename = TpaLS_LS_AbsolutePath(ls, filename);
			if (path_filename == NULL) {
				TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for pubkey failed");
				goto error;
			}
			if (Tpa_BLOB_writeToFile(path_filename,
					((TPA_GENERIC_TPMKEY *) (obj->objptr))->blob,
					((TPA_GENERIC_TPMKEY *) (obj->objptr))->blob_size)
					!= TPA_SUCCESS) {
				TRACE_MSG_ERROR("TpaHL_BLOB_writePubBlobToFile failed");
				goto error;
			}
			free(path_filename);
			path_filename = NULL;
		} else {
			TRACE_MSG_ERROR("internal error");
		}
		break;

		/****************************************************************************/
		/*																			*/
		/* 							Store BINDING KEY								*/
		/*																			*/
		/****************************************************************************/
	case DB_BINDINGKEY_TYPE:
		((TPA_TPMKEY *) (obj->objptr))->tpmkey_id = current_id;
		/* Update keys table */
		snprintf(query, MAX_QUERY_DIM, "INSERT INTO keys VALUES(%d, %d);",
				current_id, DB_BINDINGKEY_TYPE);
		/* Execute query */
		if (sqlite3_exec(ls->sqldb, query, NULL, NULL, &err) != SQLITE_OK) {
			fprintf(stderr, "SQL error: %s\n", err);
			sqlite3_free(err);
			goto error;
		}
		/* Save blobs into storage */
		snprintf(filename, MAX_LINE_SIZE, "%d"PUBKEY_SUFFIX, current_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for pubkey failed");
			goto error;
		}
		if (TpaHL_GenTpmKey_writePubKeyToFile(path_filename,
				((TPA_TPMKEY *) (obj->objptr))->pubKey,
				((TPA_TPMKEY *) (obj->objptr))->pubKey_size) != TPA_SUCCESS) {
			TRACE_MSG_ERROR("TpaHL_BLOB_writePubBlobToFile failed");
			goto error;
		}
		free(path_filename);
		path_filename = NULL;

		snprintf(filename, MAX_LINE_SIZE, "%d"PRIKEY_SUFFIX, current_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for privkey failed");
			goto error;
		}
		if (Tpa_BLOB_writeToFile(path_filename,
				((TPA_TPMKEY *) (obj->objptr))->blob,
				((TPA_TPMKEY *) (obj->objptr))->blob_size) != TPA_SUCCESS) {
			TRACE_MSG_ERROR("TpaHL_BLOB_writeBlobToFile (aik blob) failed");
			goto error;
		}
		free(path_filename);
		path_filename = NULL;
		break;

	default:
		TRACE_MSG_WARNING("Unknown Key")
		;
	}

	result = TPA_SUCCESS;
	error: /* free */
	if (mtx) {
		sqlite3_free_table(mtx);
	}
	if (mtx2) {
		sqlite3_free_table(mtx2);
	}
	if (path_filename)
		free(path_filename);

	return result;
}

/*
 * Retrieve object from database
 */
TPA_RESULT TpaLS_DB_LoadById(TPA_CONTEXT *context, TPA_GENERIC_OBJPTR *obj,
		TPA_LS *ls) {

	TPA_RESULT result = TPA_ERROR;
	int rows = 0;
	int cols = 0;
	int current_id = 0;
	char **mtx = NULL;
	char **mtx2 = NULL;
	char *err = NULL;
	char query[MAX_QUERY_DIM] = { 0 };
	char filename[MAX_LINE_SIZE] = { 0 };
	char *path_filename = NULL;

#ifndef __NO_PARAM_CHECK
	if (context == NULL) {
		TRACE_MSG_PARAM_ERROR("context is NULL");
		return TPA_ERROR;
	}
	if (obj == NULL) {
		TRACE_MSG_PARAM_ERROR("obj is NULL");
		return TPA_ERROR;
	}
	if (obj->objptr != NULL) {
		TRACE_MSG_ERROR("Key structure is not valid.");
		goto error;
	}
	if (ls == NULL) {
		TRACE_MSG_PARAM_ERROR("ls is NULL");
		return TPA_ERROR;
	}
	if (ls->sqldb == NULL) {
		TRACE_MSG_ERROR("Database is not open.");
		goto error;
	}
#endif

	snprintf(query, MAX_QUERY_DIM, "SELECT type FROM keys WHERE id=%d;",
			obj->saved_objid);
	/* Select id value */
	if (sqlite3_get_table(ls->sqldb, query, &mtx, &rows, &cols, &err)
			!= SQLITE_OK) {
		TRACE_MSG_ERROR_MSG("SQL error:", err);
		goto error;
	}

	if (rows == 0) {
		obj->saved_objid = -1;
		TRACE_MSG_INFO("key not found in db");
		result = TPA_SUCCESS;
		goto error;
	} else if (rows == 1) {
		obj->type = atoi(mtx[1]);
		current_id = obj->saved_objid;
	} else {
		TRACE_MSG_ERROR("error in db, primary key is not unique");
		goto error;
	}

	/* Retrieve key object */
	switch (obj->type) {

	/****************************************************************************/
	/*																			*/
	/* 									AIK										*/
	/*																			*/
	/****************************************************************************/
	case DB_AIK_TYPE:

		obj->objptr = (TPA_STRUCT_PTR *) calloc(1, sizeof(TPA_AIK));
		((TPA_AIK *) (obj->objptr))->aik_id = current_id;

		/* Load blobs from storage */
		snprintf(filename, MAX_LINE_SIZE, "%d"PUBKEY_SUFFIX, current_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for pubkey failed");
			goto error;
		}
		if (Tpa_BLOB_readFromFile(path_filename,
				&(((TPA_AIK *) (obj->objptr))->pubKey),
				&(((TPA_AIK *) (obj->objptr))->pubKey_size)) != TPA_SUCCESS) {
			TRACE_MSG_ERROR("TpaHL_BLOB_readFromFile failed");
			goto error;
		}
		free(path_filename);
		path_filename = NULL;

		snprintf(filename, MAX_LINE_SIZE, "%d"PRIKEY_SUFFIX, current_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for privkey failed");
			goto error;
		}
		if (Tpa_BLOB_readFromFile(path_filename,
				&(((TPA_AIK *) (obj->objptr))->keyBlob),
				&(((TPA_AIK *) (obj->objptr))->keyBlob_size)) != TPA_SUCCESS) {
			TRACE_MSG_ERROR("TpaHL_BLOB_readFromFile (aik blob) failed");
			goto error;
		}
		free(path_filename);
		path_filename = NULL;

		snprintf(filename, MAX_LINE_SIZE, "%d"CERT_SUFFIX, current_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for cert failed");
			goto error;
		}
		if (Tpa_BLOB_readFromFile(path_filename,
				&(((TPA_AIK *) (obj->objptr))->cert),
				&(((TPA_AIK *) (obj->objptr))->cert_size)) != TPA_SUCCESS) {
			TRACE_MSG_ERROR("TpaHL_BLOB_writeBlobToFile (cert) failed");
			goto error;
		}

		break;

		/****************************************************************************/
		/*																			*/
		/* 							Storage Master Key								*/
		/*																			*/
		/****************************************************************************/
	case DB_STORAGEMK_TYPE:

		obj->objptr = (TPA_STRUCT_PTR *) calloc(1, sizeof(TPA_GENERIC_TPMKEY));
		((TPA_GENERIC_TPMKEY *) (obj->objptr))->tpmkey_id = current_id;

		/* Retrieve blob from storage */
		snprintf(filename, MAX_LINE_SIZE, "%d"PRIKEY_SUFFIX, current_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for pubkey failed");
			goto error;
		}
		if (Tpa_BLOB_readFromFile(path_filename,
				&(((TPA_GENERIC_TPMKEY *) (obj->objptr))->blob),
				&(((TPA_GENERIC_TPMKEY *) (obj->objptr))->blob_size))
				!= TPA_SUCCESS) {
			TRACE_MSG_ERROR("TpaHL_BLOB_readFromFile failed");
			goto error;
		}
		break;

		/****************************************************************************/
		/*																			*/
		/* 							Load BINDING KEY								*/
		/*																			*/
		/****************************************************************************/
	case DB_BINDINGKEY_TYPE:

		obj->objptr = (TPA_STRUCT_PTR *) calloc(1, sizeof(TPA_TPMKEY));
		((TPA_TPMKEY *) (obj->objptr))->tpmkey_id = current_id;
		((TPA_TPMKEY *) (obj->objptr))->keyType = TPA_KEY_TYPE_BINDING;

		/* Load blobs from storage */
		snprintf(filename, MAX_LINE_SIZE, "%d"PUBKEY_SUFFIX, current_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for pubkey failed");
			goto error;
		}
		if (Tpa_BLOB_readFromFile(path_filename,
				&(((TPA_TPMKEY *) (obj->objptr))->pubKey),
				&(((TPA_TPMKEY *) (obj->objptr))->pubKey_size))
				!= TPA_SUCCESS) {
			TRACE_MSG_ERROR("TpaHL_BLOB_readFromFile failed");
			goto error;
		}
		free(path_filename);
		path_filename = NULL;

		snprintf(filename, MAX_LINE_SIZE, "%d"PRIKEY_SUFFIX, current_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for privkey failed");
			goto error;
		}
		if (Tpa_BLOB_readFromFile(path_filename,
				&(((TPA_TPMKEY *) (obj->objptr))->blob),
				&(((TPA_TPMKEY *) (obj->objptr))->blob_size))
				!= TPA_SUCCESS) {
			TRACE_MSG_ERROR("TpaHL_BLOB_readFromFile (aik blob) failed");
			goto error;
		}
		free(path_filename);
		path_filename = NULL;

		break;

	default:
		TRACE_MSG_WARNING("Unknown Key")
		;
	}

	result = TPA_SUCCESS;
	error: /* free */
	if (mtx) {
		sqlite3_free_table(mtx);
	}
	if (mtx2) {
		sqlite3_free_table(mtx2);
	}
	if (err) {
		sqlite3_free(err);
	}
	if (path_filename)
		free(path_filename);

	return result;

}

TPA_RESULT TpaLS_DB_LoadStorageMK(TPA_CONTEXT *context,
		TPA_GENERIC_OBJPTR *obj, TPA_LS *ls) {

	TPA_RESULT result = TPA_ERROR;
	int rows = 0;
	int cols = 0;
	//int i = 0;
	int current_id = 0;
	char **mtx = NULL;
	char *err = NULL;
	char query[MAX_QUERY_DIM] = { 0 };
	char filename[MAX_LINE_SIZE] = { 0 };
	char *path_filename = NULL;

	if (obj == NULL || obj->objptr != NULL) {
		TRACE_MSG_ERROR("Key structure is not valid.");
		goto error;
	}

	if (ls->sqldb == NULL) {
		TRACE_MSG_ERROR("Database is not open.");
		goto error;
	}

	snprintf(query, MAX_QUERY_DIM, "SELECT id FROM keys WHERE type=%d;",
			DB_STORAGEMK_TYPE);
	/* Select id value */
	if (sqlite3_get_table(ls->sqldb, query, &mtx, &rows, &cols, &err)
			!= SQLITE_OK) {
		TRACE_MSG_ERROR_MSG("SQL error:", err);
		sqlite3_free(err);
		goto error;
	}

	if (rows == 0) {
		TRACE_MSG_INFO("Storage Master Key not found in db");
		current_id = DB_KEY_NOT_FOUND;
		goto end;
	} else if (rows == 1) {
		obj->type = DB_STORAGEMK_TYPE;
		current_id = atoi(mtx[1]);
	} else {
		TRACE_MSG_ERROR("more than one Storage Master Key found on db");
		goto error;
	}

	obj->objptr = (TPA_STRUCT_PTR *) calloc(1, sizeof(TPA_GENERIC_TPMKEY));
	((TPA_GENERIC_TPMKEY *) (obj->objptr))->tpmkey_id = current_id;

	/* Retrieve blob from storage */
	snprintf(filename, MAX_LINE_SIZE, "%d"PRIKEY_SUFFIX, current_id);
	path_filename = TpaLS_LS_AbsolutePath(ls, filename);
	if (path_filename == NULL) {
		TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for pubkey failed");
		goto error;
	}
	if (Tpa_BLOB_readFromFile(path_filename,
			&(((TPA_GENERIC_TPMKEY *) (obj->objptr))->blob),
			&(((TPA_GENERIC_TPMKEY *) (obj->objptr))->blob_size))
			!= TPA_SUCCESS) {
		TRACE_MSG_ERROR("TpaHL_BLOB_readFromFile failed");
		goto error;
	}
	free(path_filename);
	path_filename = NULL;

	end: obj->saved_objid = current_id;
	result = TPA_SUCCESS;
	error: /* free */
	if (mtx) {
		sqlite3_free_table(mtx);
	}
	if (path_filename)
		free(path_filename);

	return result;
}

TPA_RESULT TpaLS_DB_LookForStorageMK(TPA_CONTEXT *context,
		TPA_GENERIC_OBJPTR *obj, TPA_LS *ls) {

	TPA_RESULT result = TPA_ERROR;
	int rows = 0;
	int cols = 0;
	//int i = 0;
	char **mtx = NULL;
	char *err = NULL;
	char query[MAX_QUERY_DIM] = { 0 };
	char *path_filename = NULL;

	if (obj == NULL) {
		TRACE_MSG_ERROR("Key structure is not valid.");
		goto error;
	}

	if (ls->sqldb == NULL) {
		TRACE_MSG_ERROR("Database is not open.");
		goto error;
	}

	snprintf(query, MAX_QUERY_DIM, "SELECT id FROM keys WHERE type=%d;",
			DB_STORAGEMK_TYPE);
	/* Select id value */
	if (sqlite3_get_table(ls->sqldb, query, &mtx, &rows, &cols, &err)
			!= SQLITE_OK) {
		TRACE_MSG_ERROR_MSG("SQL error:", err);
		sqlite3_free(err);
		goto error;
	}

	if (rows == 0) {
		TRACE_MSG_INFO("Storage Master Key not found in db");
		obj->saved_objid = DB_KEY_NOT_FOUND;
		result = TPA_SUCCESS;
		goto error;
	} else if (rows == 1) {
		obj->type = DB_STORAGEMK_TYPE;
		obj->saved_objid = atoi(mtx[1]);
	} else {
		TRACE_MSG_ERROR("more than one Storage Master Key found on db");
	}

	result = TPA_SUCCESS;
	error: /* free */
	if (mtx) {
		sqlite3_free_table(mtx);
	}
	if (path_filename)
		free(path_filename);

	return result;
}

TPA_RESULT TpaLS_DB_List(TPA_CONTEXT *context, TPA_LS *ls,
		TPA_OBJ_TYPE kType, int **key_list, int *key_list_size) {

	int i = 0;
	int row = 0;
	int col = 0;

	char **mtx = NULL;
	char *err = NULL;
	char query[MAX_QUERY_DIM] = { 0 };

#ifndef __NO_PARAM_CHECK
	if (context == NULL) {
		TRACE_MSG_PARAM_ERROR("context is NULL");
		return TPA_ERROR;
	}

	if (ls == NULL) {
		TRACE_MSG_PARAM_ERROR("ls is NULL");
		goto error;
	}

	if (ls->sqldb == NULL) {
		TRACE_MSG_PARAM_ERROR("ls->sqldb is NULL");
		goto error;
	}

	if (key_list == NULL) {
		TRACE_MSG_PARAM_ERROR("key_list is NULL");
		goto error;
	}

	if (key_list_size == 0) {
		TRACE_MSG_PARAM_ERROR("key_list_size is NULL");
		goto error;
	}

	if (kType != DB_AIK_TYPE && kType != DB_BINDINGKEY_TYPE) {
		TRACE_MSG_PARAM_ERROR("key-type is not valid");
		goto error;
	}
#endif

	snprintf(query, MAX_QUERY_DIM, "SELECT id FROM keys WHERE type=%d;", kType);

	if (sqlite3_get_table(ls->sqldb, query, &mtx, &row, &col, &err)
			!= SQLITE_OK) {
		TRACE_MSG_ERROR_MSG("SQL error:", err);
		sqlite3_free(err);
		goto error;
	}

	*key_list_size = row;
	*key_list = (int *) malloc(*key_list_size * sizeof(int));
	if (*key_list == NULL) {
		TRACE_MSG_ERROR("malloc failed");
		goto error;
	}

	for (i = 1; i <= row; i++) {
		(*key_list)[i - 1] = atoi(mtx[i]);
	}

	sqlite3_free_table(mtx);
	return TPA_SUCCESS;

	error: /* error condition */
	if (*key_list) {
		free(*key_list);
		*key_list = NULL;
	}
	*key_list_size = 0;

	if (err) {
		sqlite3_free(err);
	}

	if (mtx)
		sqlite3_free_table(mtx);

	return TPA_ERROR;
}

/*
 * Delete object from database
 */
//#define TPA_DEBUG_TRACEDELETE
TPA_RESULT TpaLS_DB_DeleteById(TPA_CONTEXT *context, TPA_OBJ_ID key_id,
		TPA_OBJ_TYPE type, TPA_LS *ls) {

	TPA_RESULT result = TPA_ERROR;
	TPA_OBJ_TYPE ktype = -1;
	int rows = 0;
	int cols = 0;
	char **mtx = NULL;
	char **mtx2 = NULL;
	char *err = NULL;
	char query[MAX_QUERY_DIM] = { 0 };
	char filename[MAX_LINE_SIZE] = { 0 };
	char *path_filename = NULL;

#ifndef __NO_PARAM_CHECK
	if (context == NULL) {
		TRACE_MSG_PARAM_ERROR("context is NULL");
		return TPA_ERROR;
	}
	if (key_id <= 0) {
		TRACE_MSG_PARAM_ERROR("key_id is not valid");
		return TPA_ERROR;
	}
	if (ls == NULL) {
		TRACE_MSG_PARAM_ERROR("ls is NULL");
		return TPA_ERROR;
	}
	if (ls->sqldb == NULL) {
		TRACE_MSG_ERROR("Database is not open.");
		goto error;
	}
#endif

	snprintf(query, MAX_QUERY_DIM, "SELECT type FROM keys WHERE id=%d;", key_id);
	/* Select id value */
	if (sqlite3_get_table(ls->sqldb, query, &mtx, &rows, &cols, &err)
			!= SQLITE_OK) {
		TRACE_MSG_ERROR_MSG("SQL error:", err);
		goto error;
	}

	if (rows == 0) {
		TRACE_MSG_INFO("key not found in db");
		result = TPA_SUCCESS;
		goto error;
	} else if (rows == 1) {
		ktype = atoi(mtx[1]);
		if (type != ktype) {
			TRACE_MSG_ERROR("key_id has a wrong type");
			goto error;

		}

#ifndef TPA_DEBUG_TRACEDELETE
		/* Delete in keys */
		snprintf(query, MAX_QUERY_DIM, "DELETE FROM keys WHERE id=%d;", key_id);
		if (sqlite3_exec(ls->sqldb, query, NULL, NULL, &err) != SQLITE_OK) {
			fprintf(stderr, "SQL error: %s\n", err);
			goto error;
		}
#else
		printf("I'm going to delete the entry in the db\n");
#endif

	} else {
		TRACE_MSG_ERROR("error in db, primary key is not unique");
		goto error;
	}

	/* Retrieve key object */
	switch (ktype) {

	/****************************************************************************/
	/*																			*/
	/* 									AIK										*/
	/*																			*/
	/****************************************************************************/
	case DB_AIK_TYPE:
		snprintf(filename, MAX_LINE_SIZE, "%d"PUBKEY_SUFFIX, key_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for pubkey failed");
			goto error;
		}
#ifndef TPA_DEBUG_TRACEDELETE
		unlink(path_filename);
#else
		printf("I'm going to delete %s\n", path_filename);
#endif
		free(path_filename);
		path_filename = NULL;
		snprintf(filename, MAX_LINE_SIZE, "%d"PRIKEY_SUFFIX, key_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for privkey failed");
			goto error;
		}
#ifndef TPA_DEBUG_TRACEDELETE
		unlink(path_filename);
#else
		printf("I'm going to delete %s\n", path_filename);
#endif
		free(path_filename);
		path_filename = NULL;
		snprintf(filename, MAX_LINE_SIZE, "%d"CERT_SUFFIX, key_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for cert failed");
			goto error;
		}
#ifndef TPA_DEBUG_TRACEDELETE
		unlink(path_filename);
#else
		printf("I'm going to delete %s\n", path_filename);
#endif
		break;

		/****************************************************************************/
		/*																			*/
		/* 							Storage Master Key								*/
		/*																			*/
		/****************************************************************************/
	case DB_STORAGEMK_TYPE:
		snprintf(filename, MAX_LINE_SIZE, "%d"PRIKEY_SUFFIX, key_id);
		path_filename = TpaLS_LS_AbsolutePath(ls, filename);
		if (path_filename == NULL) {
			TRACE_MSG_ERROR("TpaLS_LS_AbsolutePath for pubkey failed");
			goto error;
		}
#ifndef TPA_DEBUG_TRACEDELETE
		unlink(path_filename);
#else
		printf("I'm going to delete %s\n", path_filename);
#endif
		break;

	default:
		TRACE_MSG_WARNING("Unknown Key")
		;
	}

	result = TPA_SUCCESS;
	error: /* free */
	if (mtx) {
		sqlite3_free_table(mtx);
	}
	if (mtx2) {
		sqlite3_free_table(mtx2);
	}
	if (err) {
		sqlite3_free(err);
	}
	if (path_filename)
		free(path_filename);

	return result;
}

TPA_RESULT TpaLS_DB_Is(TPA_OBJ_TYPE type, TPA_OBJ_ID *id, int *result) {
	return TPA_ADV_FUNCTION;
}

/********************************************************************************/
/*																				*/
/*								OLD FUNCTIONS									*/
/*																				*/
/********************************************************************************/

/*
 * STORE OLD
 */
TPA_RESULT TpaLS_AIK_DB_Store(TPA_CONTEXT *context, TPA_AIK *aik,
		TPA_LS *ls) {
	printf("Hi, I'm Tpa_AIK_DB_Store stub, and I'm an advanced function!\n");
	return TPA_ADV_FUNCTION;
}

/*
 * RETRIEVE
 */
TPA_RESULT TpaLS_AIK_DB_Retrieve(TPA_CONTEXT *context, TPA_AIK *aik,
		TPA_LS *ls) {
	printf("Hi, I'm Tpa_AIK_DB_Retrieve stub, and I'm an advanced function!\n");
	return TPA_ADV_FUNCTION;
}

/*
 * DELETE
 */
TPA_RESULT TpaLS_AIK_DB_Delete(TPA_CONTEXT *context, TPA_AIK *aik,
		TPA_LS *ls) {
	printf("Hi, I'm Tpa_AIK_DB_Retrieve stub, and I'm an advanced function!\n");
	return TPA_ADV_FUNCTION;
}

/*
 * LIST
 */
TPA_RESULT TpaLS_AIK_DB_List(TPA_CONTEXT *context, TPA_AIK **aikList, int *len,
		TPA_LS *ls) {
	printf("Hi, I'm Tpa_AIK_DB_List stub, and I'm an advanced function!\n");
	return TPA_ADV_FUNCTION;
}

TPA_RESULT TpaLS_DB_ListTest(TPA_CONTEXT *context, TPA_AIK **aikList, int *len,
		TPA_LS *ls) {

	int i = 0;
	int j = 0;
	int row = 0;
	int col = 0;

	char **mtx = NULL;
	char *err = NULL;

	sqlite3_get_table(ls->sqldb, "SELECT * FROM keys;", &mtx, &row, &col, &err);

	for (j = 0; j < col; j++) {
		printf("%s | ", mtx[j]);
	}
	printf("\n");
	printf("\n");

	for (i = 1; i < row; i++) {
		//		for (j=0; j<col; j++) {
		//			printf("%s | ", mtx[i*row+j]);
		//		}

		printf("%s | ", mtx[i * col]);

		switch (atoi(mtx[i * col + 1])) {
		case DB_AIK_TYPE:
			printf("AIK");
			break;
		case DB_STORAGEMK_TYPE:
			printf("Storage Master Key");
			break;

		default:
			printf("Unknown");
		}

		printf("\n");
	}

	printf("\n");

	return TPA_SUCCESS;
}

TPA_RESULT TpaLS_Cert_DB_Store(TPA_CONTEXT *context, TPA_CERTIFICATE_ITEM *aik,
		TPA_LS *ls) {
	printf("Hi, I'm TpaLS_Cert_DB_Store stub, and I'm an advanced function!\n");
	return TPA_ADV_FUNCTION;
}

#endif
