#include "TPA_TEST.h"

void _bind_test_success(void **state) {
	TPA_PCR *pcr = NULL;

	assert_int_equal(
		TpaHL_PCR_allocate(&pcr), TPA_SUCCESS );

	assert_int_equal( pcr->pcr,         0 );
	assert_int_equal( pcr->pcr_size,    20 );
	assert_int_equal( (int) pcr->value, (int) NULL );

	assert_int_equal(
		TpaHL_PCR_set(pcr, 0, 0, NULL), TPA_NOT_IMPLEMENTED );

	TpaHL_PCR_freeMemory(pcr);
}


void bind_test_success(void **state) {
	int test_finished = 0;

	TPA_CONTEXT *ctx = NULL;
	TPA_TPM *tpm = NULL;
	TPA_TPMKEY *key = NULL;

	unsigned char *bind_blob = NULL;
	unsigned int bind_blob_len = 0;
	unsigned char *verify_blob = NULL;
	unsigned int verify_blob_len = 0;

	/* Clear local storage dir */
	clear_LS();

	/* Allocate all needed objects */
	if (TpaHL_CTX_allocate(&ctx) != TPA_SUCCESS) {
		goto error;
	}

	if (TpaHL_TPM_allocate(&tpm) != TPA_SUCCESS) {
		goto error;
	}

	if (TpaHL_KEY_allocate(&key) != TPA_SUCCESS) {
		goto error;
	}

	if (TpaHL_CTX_set(ctx, CTX_LS_URI, strlen(TPA_DEFAULT_LOCAL_STORAGE), TPA_DEFAULT_LOCAL_STORAGE) != TPA_SUCCESS) {
		goto error;
	}

	/* Create Local Storage */	
	if (TpaHL_LS_create(ctx) != TPA_SUCCESS) {
		goto error;
	}
	
	if (TpaHL_TPM_set(tpm, TPM_SRKPWD, strlen(TPA_TPM_DEFAULT_SRKPWD), TPA_TPM_DEFAULT_SRKPWD) != TPA_SUCCESS) {
		goto error;
	}
	
	/* Create Storage Master Key */
	if (TpaHL_SMK_Create(ctx, tpm) != TPA_SUCCESS) {
		goto error;
	}

	/* Create Key */
	if (TpaHL_KEY_set(key, KEY_TPMKEYSECRET, strlen(TPA_TPM_DEFAULT_KEYPWD), TPA_TPM_DEFAULT_KEYPWD) != TPA_SUCCESS) {
		goto error;
	}
	
	if (TpaHL_KEY_CreateTpmBindingKey(ctx, tpm, key) != TPA_SUCCESS) {
		goto error;
	}

	/* Perform Binding */		
	if (TpaHL_DATA_BindBlob(ctx, tpm, key, (unsigned char *) TPA_DEFAULT_TEST_STRING, strlen(TPA_DEFAULT_TEST_STRING),
			&bind_blob, &bind_blob_len) != TPA_SUCCESS) {
		goto error;
	}

	/* Perform Unbinding */
	if (TpaHL_DATA_UnbindBlob(ctx, tpm, key, bind_blob, bind_blob_len,
			&verify_blob, &verify_blob_len) != TPA_SUCCESS) {
		goto error;
	}
	
	/* Verify binding/unbinding results */
	assert_int_equal (strlen(TPA_DEFAULT_TEST_STRING), verify_blob_len);
	assert_memory_equal (TPA_DEFAULT_TEST_STRING, verify_blob, verify_blob_len);

	/* Test finished */
	test_finished = 1;

	error: /* free */
	TpaHL_KEY_freeMemory(key);
	TpaHL_TPM_freeMemory(tpm);
	TpaHL_CTX_freeMemory(ctx);

	/* Clear local storage dir */
	clear_LS();

	assert_true (test_finished);
}
